/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */ 
package com.ibm.wbimonitor.xml.model.mm.util;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xml.type.internal.QName;

import com.ibm.wbimonitor.xml.model.mm.EventPartType;
import com.ibm.wbimonitor.xml.model.mm.ImportType;

/**
 * <samp>IEventTypeResolver</samp>
 * Resolves the type of the given event. 
 */
public interface IEventTypeResolver {
    public static final String COPYRIGHT = "(C) Copyright IBM Corporation 2007.";  //$NON-NLS-1$

	
	/**
	 * This method resolves a QName reference from an EventPart to the actual XSDTypeDefinition
	 * object that is referenced.
	 * 
	 * @param parent The EventPartType object that references the type definition
	 * @param typeQName is the QName of the desired type
	 * @return the {@link org.eclipse.xsd.XSDTypeDefinition XSD type definition}
	 */
	public EObject resolve(EventPartType parent, QName typeQName);

	/**
	 * Returns whether this event type resolver is capable of resolving the given 
	 * import type.
	 * 
	 * @param importType The MM import statement that needs resolving
	 * @return True if this IEventTypeResolver can resolve the import type, false otherwise
	 */
	public boolean isApplicableForImport(ImportType importType);
	
	/**
	 * Resolves the import type to the object to which it points.  Each subclass of IEventTypeResolver
	 * may return a different type of object, depending on the type of object being resolved
	 * 
	 * @param importType The import statement that requires resolution
	 * @return The result of the resolving the import type.
	 */
	public Object resolveImport(ImportType importType);
}
