/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2006, 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.wbimonitor.xml.model.mm.util;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.emf.ecore.xmi.XMLHelper;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.SAXXMLHandler;

import com.ibm.wbimonitor.xml.model.mm.MmPackage;
import com.ibm.wbimonitor.xml.model.mm.NamedElementType;

/**
 * 
 */
public class MmSAXXMLHandler extends SAXXMLHandler {
public static final String COPYRIGHT="(C) Copyright IBM Corporation 2006, 2007.";  //$NON-NLS-1$


	/**
	 * @param xmiResource
	 * @param helper
	 * @param options
	 */
	public MmSAXXMLHandler(XMLResource xmiResource, XMLHelper helper, Map options) {
		super(xmiResource, helper, options);
	}
	
	  /**
	 * Check if the values of the forward references have been set (they may
	 * have been set due to a bi-directional reference being set).  If not,
	 * set them.
	 * If this is called during end document processing, errors should be diagnosed.
	 * If it is called in the middle of a document, 
	 * we need to clean up the forward reference lists to avoid reprocessing resolved references again later.
	 */
	protected void handleForwardReferences(boolean isEndDocument) {
		// Handle the same document proxies, which may have problems resulting from the
		// other end of a bidirectional reference being handled as an IDREF rather than as a proxy.
		// When we are done with these, we know that funny proxies are now resolved as if they were handled as IDREFs.
		//
		for (Iterator i = sameDocumentProxies.iterator(); i.hasNext();) {
			InternalEObject proxy = (InternalEObject) i.next();

			// Look through all the references...
			//
			LOOP: for (Iterator j = proxy.eClass().getEAllReferences()
					.iterator(); j.hasNext();) {
				// And find the one that holds this proxy.
				//
				EReference eReference = (EReference) j.next();
				EReference oppositeEReference = eReference.getEOpposite();
				if (oppositeEReference != null && proxy.eIsSet(eReference)) {
					// Try to resolve the proxy locally.
					//
					EObject resolvedEObject = xmlResource.getEObject(proxy
							.eProxyURI().fragment());
					if (resolvedEObject != null) {
						// We won't need to process this again later.
						//
						if (!isEndDocument) {
							i.remove();
						}

						// Compute the holder of the proxy
						//
						EObject proxyHolder = (EObject) (eReference.isMany() ? ((List) proxy
								.eGet(eReference)).get(0)
								: proxy.eGet(eReference));

						// If the proxy holder can hold many values,
						// it may contain a duplicate that resulted when the other end was processed as an IDREF
						// and hence did both sides of the bidirectional relation.
						//
						if (oppositeEReference.isMany()) {
							// So if the resolved object is also present...
							//
							InternalEList holderContents = (InternalEList) proxyHolder
									.eGet(oppositeEReference);
							List basicHolderContents = holderContents
									.basicList();
							int resolvedEObjectIndex = basicHolderContents
									.indexOf(resolvedEObject);
							if (resolvedEObjectIndex != -1) {
								// Move the resolved object to the right place, remove the proxy, and we're done.
								//
								int proxyIndex = basicHolderContents
										.indexOf(proxy);
								holderContents.move(proxyIndex,
										resolvedEObjectIndex);
								holderContents
										.remove(proxyIndex > resolvedEObjectIndex ? proxyIndex - 1
												: proxyIndex + 1);
								break LOOP;
							}
						}

						// If the resolved object doesn't contain a reference to the proxy holder as it should.
						//
						if (eReference.isMany() ? !((InternalEList) resolvedEObject
								.eGet(eReference)).basicList().contains(
								proxyHolder)
								: resolvedEObject.eGet(eReference) != proxyHolder) {
							// The proxy needs to be replaced in a way that updates both ends of the reference.
							//
							if (oppositeEReference.isMany()) {
								InternalEList proxyHolderList = (InternalEList) proxyHolder
										.eGet(oppositeEReference);
								proxyHolderList.setUnique(proxyHolderList
										.indexOf(proxy), resolvedEObject);
							} else {
								proxyHolder.eSet(oppositeEReference,
										resolvedEObject);
							}
						}
					}

					break;
				}
			}
		}

		for (Iterator i = forwardSingleReferences.iterator(); i.hasNext();) {
			SingleReference ref = (SingleReference) i.next();
			EObject obj = xmlResource.getEObject((String) ref.getValue());

			if (obj != null) {
				// We won't need to process this again later.
				if (!isEndDocument) {
					i.remove();
				}
				EStructuralFeature feature = ref.getFeature();
				setFeatureValue(ref.getObject(), feature, obj, ref
						.getPosition());
			} else if (isEndDocument) {
				// We don't find the object so it is a broken ref and we create a proxy
				EStructuralFeature feature = ref.getFeature();
				
				/* The reference is likely of type NamedElementType, which is abstract, so create a Metric as a concrete 
				 * implementation of a NamedElementType proxy */
				//NamedElementType proxy = (NamedElementType)MmPackage.eINSTANCE.getMmFactory().createMetricType();
				NamedElementType proxy = (NamedElementType)MmPackage.eINSTANCE.getMmFactory().create((EClass)feature.getEType());
				URI proxyURI = xmlResource.getURI().appendFragment((String)ref.getValue());
				((InternalEObject)proxy).eSetProxyURI(proxyURI);
				setFeatureValue(ref.getObject(), feature, proxy, ref.getPosition());				
			}
		}

		for (Iterator i = forwardManyReferences.iterator(); i.hasNext();) {
			ManyReference ref = (ManyReference) i.next();
			Object[] values = ref.getValues();

			boolean failure = false;
			for (int j = 0, l = values.length; j < l; j++) {
				String id = (String) values[j];
				EObject obj = xmlResource.getEObject(id);
				
				if(obj != null){
					values[j] = obj;
				}
				else if (isEndDocument) {
					EStructuralFeature feature = ref.getFeature();
					NamedElementType proxy = (NamedElementType)MmPackage.eINSTANCE.getMmFactory().create((EClass)feature.getEType());
					//NamedElementType proxy = (NamedElementType)MmPackage.eINSTANCE.getMmFactory().createMetricType(); //((EClass)feature.getEType());
					URI proxyURI = xmlResource.getURI().appendFragment(id);
					((InternalEObject)proxy).eSetProxyURI(proxyURI);
					values[j] = proxy;
				}
			}

			if (!failure) {
				if (!isEndDocument) {
					i.remove();
				}
				setFeatureValues(ref);
			} else if (isEndDocument) {
				// At least set the references that we were able to resolve, if any.
				//
				setFeatureValues(ref);
			}
		}
	}	


	/* (non-Javadoc)
	 * @see org.eclipse.emf.ecore.xmi.impl.XMLHandler#processObject(org.eclipse.emf.ecore.EObject)
	 */
	protected void processObject(EObject object) {
		if (object != null) {
			int lineNumber = getLineNumber();
			Resource resource = helper.getResource();
			if (resource instanceof MmResourceImpl) {
				((MmResourceImpl) resource).registerLineNumebr(object, lineNumber);
			}
		}
		super.processObject(object);
	}
}
