/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2006, 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.wbimonitor.xml.model.util;

import org.eclipse.emf.ecore.EObject;

import com.ibm.wbimonitor.xml.model.mm.CubeType;
import com.ibm.wbimonitor.xml.model.mm.DimensionAttributeType;
import com.ibm.wbimonitor.xml.model.mm.DimensionalModelType;
import com.ibm.wbimonitor.xml.model.mm.MeasureType;
import com.ibm.wbimonitor.xml.model.mm.MonitorType;
import com.ibm.wbimonitor.xml.model.mm.ReferenceType;

public class DataMartModelContextResolver extends ModelContextResolver {
public static final String COPYRIGHT="(C) Copyright IBM Corporation 2006, 2007.";  //$NON-NLS-1$

	
	private static DataMartModelContextResolver sInstance;
	
	private DataMartModelContextResolver(){
	}
	
	public static DataMartModelContextResolver getInstance(){
		if (sInstance == null){
			sInstance = new DataMartModelContextResolver();
		}
		return sInstance;
	}
	
	/**
	 * This implementation of the getContextObject() method takes advantage
	 * of the fact that all references from objects inside the DataMartModel
	 * have the enclosing Cube as their source object
	 */
	public EObject getContextObject(EObject source){
		
		EObject context = getSpecialContext(source);
		
		if (context != null){
			return context;
		}
		
		EObject parent = source;
		while (parent != null && !(parent instanceof CubeType)){
			parent = parent.eContainer();
		}
		
		if (parent instanceof CubeType){
			return ((CubeType)parent);
		}
		
		return null;
		
	}
	
	public EObject getContextObject(EObject source, EObject target){
		EObject context = null;
	
		context = getSpecialContext(source);
	
		if (context == null){			
			context = super.getContextObject(source, target);
		}
		return context;
	}
	
	private EObject getSpecialContext(EObject source){
		EObject context = null;
		
		if (source instanceof ReferenceType){
			source = source.eContainer();
		}
	
		
		if (source instanceof CubeType){
			/* For CubeType --> MonitoringContextType, context = MonitoringModel */
			context = ((MonitorType)((DimensionalModelType)getModel(source)).eContainer()).getMonitorDetailsModel();
			
		} else if (source instanceof MeasureType){
			/* For MeasureType.source --> BaseMetricType, context = the MC associated with the containing cube */ 
			context = ((CubeType)source.eContainer()).getMonitoringContextObject();
		
		} else if (source instanceof DimensionAttributeType){
		
			/* First, normalize the source to the Cube */
			source = source.eContainer().eContainer();
		
			context = ((CubeType)source).getMonitoringContextObject();
		}		
		
		return context;
	}
	
}
