/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2006, 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.wbimonitor.xml.model.util;

import org.eclipse.emf.ecore.EObject;

import com.ibm.wbimonitor.xml.model.mm.BaseMetricType;
import com.ibm.wbimonitor.xml.model.mm.ContextType;
import com.ibm.wbimonitor.xml.model.mm.CubeType;
import com.ibm.wbimonitor.xml.model.mm.KPIAggregatedDefinitionType;
import com.ibm.wbimonitor.xml.model.mm.KPIDateTimeMetricFilterRefType;
import com.ibm.wbimonitor.xml.model.mm.KPIType;
import com.ibm.wbimonitor.xml.model.mm.MeasureType;
import com.ibm.wbimonitor.xml.model.mm.MmPackage;
import com.ibm.wbimonitor.xml.model.mm.MonitorType;
import com.ibm.wbimonitor.xml.model.mm.MonitoringContextType;
import com.ibm.wbimonitor.xml.model.mm.ReferenceType;
import com.ibm.wbimonitor.xml.model.mm.impl.KPIMetricFilterRefTypeImpl;

public class KPIModelContextResolver extends ModelContextResolver {
public static final String COPYRIGHT="(C) Copyright IBM Corporation 2006, 2007.";  //$NON-NLS-1$

	
	private static KPIModelContextResolver sInstance;
	
	private KPIModelContextResolver(){
	}
	
	public static KPIModelContextResolver getInstance(){
		if (sInstance == null){
			sInstance = new KPIModelContextResolver();
		}
		return sInstance;
	}
	
	public EObject getContextObject(EObject source, EObject target){
		
		EObject context = getSpecialContext(source, target);
		
		if (context == null){			
			context = super.getContextObject(source, target);
		}
		return context;
	}
	
	/**
	 * This implementation of the getContextObject() method takes advantage
	 * of the fact that all references from objects inside the MonitoringModel
	 * have the enclosing Context as their source object, be it a Monitoring Context
	 * or a KPI Context.
	 */
	public EObject getContextObject(EObject source){
		
		EObject context = getSpecialContext(source, null);
		
		if (context != null){
			return context;
		}
		
		EObject parent = source;
		while (parent != null && !(parent instanceof ContextType)){
			parent = parent.eContainer();
		}
		
		if (parent instanceof ContextType){
			return (ContextType)parent;
		}
		
		return null;
		
	}
	
	private EObject getSpecialContext(EObject source, EObject target){
		EObject context = null;
		
		/* source is originally the KPIType... */
//		source = ((KPIType)source).getAggregatedDefinition();
		
		/* References to handle: 
		 * CASE 1: <kpi><aggregatedDefinition><metric ref="XXX"> --> BaseMetricType  			      (context = MC specified in the <monitoringContext> reference)
		 * CASE 2: <kpi><aggregatedDefinition><metricFilter ref="XXX"> --> BaseMetricType  		      (context = MC specified in the <monitoringContext> reference)
		 * CASE 3: <kpi><aggregatedDefinition><dateTimeMetricFilter ref="XXX"> --> BaseMetricType  	  (context = MC specified in the <monitoringContext> reference)
		 * CASE 4: <kpi><aggregatedDefinition><monitoringContext ref="XXX"> --> MonitoringContextType (context = MonitorDetailsModel)
		 **/
				
		if (source instanceof KPIType && target instanceof MonitoringContextType){
			context = getModel(target);
		} else if (source instanceof KPIType && target instanceof BaseMetricType){
			context = ((KPIType)source).getAggregatedDefinition().getMonitoringContext().getRefObject();			

		} else if (source instanceof KPIMetricFilterRefTypeImpl || source instanceof KPIDateTimeMetricFilterRefType
				|| (source instanceof ReferenceType && source.eContainingFeature().equals(MmPackage.eINSTANCE.getKPIAggregatedDefinitionType_Metric()))){
			// CASE 1, 2, 3

			KPIAggregatedDefinitionType aggDefType = (KPIAggregatedDefinitionType)source.eContainer();
			if (aggDefType.getMonitoringContext() != null && aggDefType.getMonitoringContext().getRefObject() instanceof MonitoringContextType){ 
				context = aggDefType.getMonitoringContext().getRefObject();
			}
			
		} else if (source instanceof ReferenceType && source.eContainingFeature().equals(MmPackage.eINSTANCE.getKPIAggregatedDefinitionType_MonitoringContext())){
			// CASE 4
			KPIAggregatedDefinitionType aggDefType = (KPIAggregatedDefinitionType)source.eContainer();
			context = ((MonitorType)getModel(source).eContainer()).getMonitorDetailsModel();
		}
		
		return context;
			
	}
			
	
}
