/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2006, 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.wbimonitor.xml.model.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.jxpath.DynamicPropertyHandler;
import org.eclipse.emf.ecore.EObject;

import com.ibm.wbimonitor.xml.model.eventdefinition501.EventDefinitionType;
import com.ibm.wbimonitor.xml.model.eventdefinition501.ExtendedDataElementType;
import com.ibm.wbimonitor.xml.model.eventdefinition501.PropertyType;
import com.ibm.wbimonitor.xml.model.mm.InboundEventType;
import com.ibm.wbimonitor.xml.model.mm.NamedElementType;
import com.ibm.wbimonitor.xml.model.mm.OutboundEventType;



public class MmDynamicPropertyHandler implements DynamicPropertyHandler {
public static final String COPYRIGHT="(C) Copyright IBM Corporation 2006, 2007.";  //$NON-NLS-1$


		public MmDynamicPropertyHandler() {
			super();
		}

		/* (non-Javadoc)
		 * @see org.apache.commons.jxpath.DynamicPropertyHandler#getProperty(java.lang.Object, java.lang.String)
		 */
		public Object getProperty(Object object, String propertyName) {

			if (object instanceof MmDynamicContextBean) {
				return getProperty((MmDynamicContextBean) object, propertyName);
			}
			return null;
		}
		/* (non-Javadoc)
		 * @see org.apache.commons.jxpath.DynamicPropertyHandler#getPropertyNames(java.lang.Object)
		 */
		public String[] getPropertyNames(Object object) {
			if (object instanceof MmDynamicContextBean) {
				return getPropertyNames((MmDynamicContextBean) object);
			}
			return new String[0];
		}
		
		public MmDynamicContextBean getProperty(MmDynamicContextBean object, String propertyName) {
			Object contextObject = object.getContextObject();
			Object eventContextObject = null;
			
			if (contextObject instanceof InboundEventType) {
				// Redirect the context object to the event definition type.
				EObject type = ((InboundEventType) contextObject).getEventTypeObject();
				if (type instanceof EventDefinitionType) {
					eventContextObject = type;
				}
			} else if (contextObject instanceof OutboundEventType) {
				// Redirect the context object to the event definition type.
				EObject type = ((OutboundEventType) contextObject).getEventTypeObject();
				if (type instanceof EventDefinitionType) {
					eventContextObject = type;
				}
			}
			
			if (eventContextObject instanceof EventDefinitionType) {
				List children = new ArrayList();
				if (! object.isStaticStep()) {
					children.add(PathExpressionUtil.CONTEXT_DATA_STEP);
					children.add(PathExpressionUtil.EXTENDED_DATA_STEP);
				} else if (object.getStaticStep().equals(PathExpressionUtil.CONTEXT_DATA_STEP)) {
					children.addAll((EventDefinitionParentResolver.getAllProperties((EventDefinitionType) eventContextObject)));
							
				} else if (object.getStaticStep().equals(PathExpressionUtil.EXTENDED_DATA_STEP)) {
					children.addAll((EventDefinitionParentResolver.getAllExtendedDataElements((EventDefinitionType) eventContextObject)));
					
				}
				for (Iterator it = children.iterator(); it.hasNext();) {
					Object child = it.next();
					if (child instanceof String) {
						String name = (String) child;
						name = NCNameConverter.stringToNcname(name);
						if (propertyName.equals(name)) {
							MmDynamicContextBean bean = new MmDynamicContextBean(eventContextObject);
							bean.setStaticStep(propertyName);
							return bean;
						}
						
					} else if (child instanceof PropertyType) {
						String name = ((PropertyType) child).getName();
						name = NCNameConverter.stringToNcname(name);
						if (propertyName.equals(name)) {
							return new MmDynamicContextBean(child);
						}
					} else if (child instanceof ExtendedDataElementType) {
						String name = ((ExtendedDataElementType) child).getName();
						name = NCNameConverter.stringToNcname(name);
						if (propertyName.equals(name)) {
							return new MmDynamicContextBean(child);
						}
					}
				}
			} 
			
			/* If we're here, then either eventContextObject null, or eventContextObject was NOT null but the 
			 * property was not found within the CBE, which means it's possible that the property
			 * refers to an event part, which is a named element and can be handled normally below */
			if (contextObject instanceof EObject) {
				List contentList = ((EObject) contextObject).eContents();
				for (Iterator it = contentList.iterator(); it.hasNext();) {
					Object child = it.next();
					if (child instanceof NamedElementType) {
						String name = ((NamedElementType) child).getId();
//						name = NCNameConverter.stringToNcname(name);
						if (propertyName.equals(name)) {
							return new MmDynamicContextBean(child);
						}
					} else if (child instanceof EventDefinitionType) {
						String name = ((EventDefinitionType) child).getName();
						name = NCNameConverter.stringToNcname(name);
						if (propertyName.equals(name)) {
							return new MmDynamicContextBean(child);
						}
					} else if (child instanceof ExtendedDataElementType) {
						String name = ((ExtendedDataElementType) child).getName();
						name = NCNameConverter.stringToNcname(name);
						if (propertyName.equals(name)) {
							return new MmDynamicContextBean(child);
						}
					}
				}
			}

			return null;
		}

		public String[] getPropertyNames(MmDynamicContextBean object) {
			Object contextObject = object.getContextObject();
			
			if (contextObject instanceof InboundEventType) {
				// Redirect the context object to the event definition type.
				EObject type = ((InboundEventType) contextObject).getEventTypeObject();
				if (type instanceof EventDefinitionType) {
					contextObject = type;
				}
			} else if (contextObject instanceof OutboundEventType) {
				// Redirect the context object to the event definition type.
				EObject type = ((OutboundEventType) contextObject).getEventTypeObject();
				if (type instanceof EventDefinitionType) {
					contextObject = type;
				}
			}

			Set result = new HashSet();
			String name = null;
			if (contextObject instanceof EventDefinitionType) {
				List children = new ArrayList();
				if (! object.isStaticStep()) {
					children.add(PathExpressionUtil.CONTEXT_DATA_STEP);
					children.add(PathExpressionUtil.EXTENDED_DATA_STEP);
				} else if (object.getStaticStep().equals(PathExpressionUtil.CONTEXT_DATA_STEP)) {
					children.addAll((EventDefinitionParentResolver.getAllProperties((EventDefinitionType) contextObject)));
					
				} else if (object.getStaticStep().equals(PathExpressionUtil.EXTENDED_DATA_STEP)) {
					children.addAll((EventDefinitionParentResolver.getAllExtendedDataElements((EventDefinitionType) contextObject)));					
				}
				for (Iterator it = children.iterator(); it.hasNext();) {
					Object child = it.next();
					if (child instanceof String) {
						result.add(child);
					} else if (child instanceof PropertyType) {
						name = ((PropertyType) child).getName();
						if (name != null) {
							result.add(name);
						}
					} else if (child instanceof ExtendedDataElementType) {
						name = ((ExtendedDataElementType) child).getName();
						if (name != null) {
							result.add(name);
						}
					}
				}
			} else if (contextObject instanceof EObject) {
				List contentList = ((EObject) contextObject).eContents();
				for (Iterator it = contentList.iterator(); it.hasNext();) {
					Object child = it.next();
					if (child instanceof NamedElementType) {
						name = ((NamedElementType) child).getId();
						if (name != null) {
							result.add(name);
						}
					} else if (child instanceof EventDefinitionType) {
						name = ((EventDefinitionType) child).getName();
						if (name != null) {
							result.add(name);
						}
					} else if (child instanceof ExtendedDataElementType) {
						name = ((ExtendedDataElementType) child).getName();
						if (name != null) {
							result.add(name);
						}
					}
				}
			}
			

			String[] array = (String[]) result.toArray(new String[result.size()]);
			return array;
		}
		

		/* (non-Javadoc)
		 * @see org.apache.commons.jxpath.DynamicPropertyHandler#setProperty(java.lang.Object, java.lang.String, java.lang.Object)
		 */
		public void setProperty(Object object, String propertyName, Object value) {
			// Ignore, read-only.

		}
}