/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2006, 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
/*
 * Created on 21-Jul-2006
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package com.ibm.wbimonitor.xml.model.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.EObjectImpl;

import com.ibm.wbimonitor.xml.model.eventdefinition501.EventDefinitionListType;
import com.ibm.wbimonitor.xml.model.eventdefinition501.EventDefinitionType;
import com.ibm.wbimonitor.xml.model.eventdefinition501.ExtendedDataElementType;
import com.ibm.wbimonitor.xml.model.eventdefinition501.PropertyType;
import com.ibm.wbimonitor.xml.model.mm.DocumentRoot;
import com.ibm.wbimonitor.xml.model.mm.NamedElementType;

/**
 * @author andyfu
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class PathExpressionHelper {
public static final String COPYRIGHT="(C) Copyright IBM Corporation 2006, 2007.";  //$NON-NLS-1$


	/**
	 * Tests whether an Path expression can be built with the given context and target.
	 * @param context
	 * @param target
	 * @return
	 */
	static boolean isValid(EObject context, EObject target) {
		if (context == null || target == null) {
			return false;
		}
		EObject commonAncestor = getCommonAncestor(context, target);
		if (commonAncestor == null) {
			return false;
		}
		
		EObject root = getAbsolutePathContextRoot(context);
		if (root == null) {
			return false;
		}
		
		return true;
	}
	
	
	/**
	 * Returns a list of EObjects beginning with the context and ending with the target,
	 * including all the intermedicate EObjects in between.
	 * @param context
	 * @param target
	 * @return
	 */
	static List collectIntermediateEObjects(EObject context, EObject target) {
		List result = new ArrayList();
		if (context == target) {
			// context equals target.
			result.add(context);
			result.add(target);
		} else if (isAncestor(context, target)) {
			// context is an ancestor of target.
			EObject current = target;
			while (current != null && current != context) {
				result.add(0, current);
				current = current.eContainer();
			}
			result.add(0, context);
		} else if (isAncestor(target, context)) {
			// target is an ancestor of context.
			EObject current = context;
			while (current != null && current != target) {
				result.add(current);
				current = current.eContainer();
			}
			result.add(target);
		} else {
			// Find the common ancestor. then merge the list between context->ancestor and ancestor->target.
			EObject commonAncestor = getCommonAncestor(context, target);
			if (commonAncestor != null) {
				List context2CommonAncestor = collectIntermediateEObjects(context, commonAncestor);
				List commonAncestor2Target = collectIntermediateEObjects(commonAncestor, target);
				result.addAll(context2CommonAncestor);
				commonAncestor2Target.remove(commonAncestor);
				result.addAll(commonAncestor2Target);
			}
		}
		return result;
	}
	
	static String generatorPathExpressionStep(EObject context, EObject target) {
		if (context != null && target != null) {
			if (context == target){
				// target equals context.
				return PathExpressionUtil.SELF_STEP;
			} else if (context.eContainer() == target) {
				// target is a parent of context.
				return PathExpressionUtil.PARENT_STEP;
			} else if (target.eContainer() == context){
				// target is a child of context.
				if (target instanceof NamedElementType) {
					String step = ((NamedElementType) target).getId();
					if (step == null) {
						EStructuralFeature feature = target.eContainingFeature();
						step = ((EObjectImpl)target).eURIFragmentSegment(feature, target);
					}
					return step;
				} else if (target instanceof PropertyType) {
					StringBuffer buffer = new StringBuffer();
					
					// for properties inherited from base event, uses "predefinedData"; otherwise use "propertyData" 
					if("event".equals(((EventDefinitionType)context).getName())){
						buffer.append(PathExpressionUtil.PREDEFINED_DATA_STEP);
					}
					else{
						buffer.append(PathExpressionUtil.CONTEXT_DATA_STEP);
					}
					buffer.append(PathExpressionUtil.STEP_SEPARATOR);
					
					String step = ((PropertyType) target).getName();
					if (step != null) {
						if (! NCNameConverter.isValidNCName(step)) {
							step = quoteNonNCName(step);
						}
					} else {
						EStructuralFeature feature = target.eContainingFeature();
						step = ((EObjectImpl)target).eURIFragmentSegment(feature, target);						
					}
					buffer.append(step);
					return buffer.toString();
				} else if (target instanceof ExtendedDataElementType) {
					StringBuffer buffer = new StringBuffer();
					if (target.eContainer() instanceof EventDefinitionType) {
						buffer.append(PathExpressionUtil.EXTENDED_DATA_STEP);
						buffer.append(PathExpressionUtil.STEP_SEPARATOR);
					}
					String step = ((ExtendedDataElementType) target).getName();
					if (step != null) {
						if (! NCNameConverter.isValidNCName(step)) {
							step = quoteNonNCName(step);
						}
					} else {
						EStructuralFeature feature = target.eContainingFeature();
						step = ((EObjectImpl)target).eURIFragmentSegment(feature, target);						
					}
					buffer.append(step);
					return buffer.toString();
				} else if (target instanceof EventDefinitionType) {
					String step = ((EventDefinitionType) target).getName();
					if (step != null) {
						if (! NCNameConverter.isValidNCName(step)) {
							step = quoteNonNCName(step);
						}
					} else {
						EStructuralFeature feature = target.eContainingFeature();
						step = ((EObjectImpl)target).eURIFragmentSegment(feature, target);						
					}
					
					return step;
				}
			}
		}
		return null;
	}
	
	private static String quoteNonNCName(String str) {
		StringBuffer buffer = new StringBuffer();
		buffer.append(PathExpressionUtil.SINGLE_QUOTE);
		buffer.append(escapeStringLiteral(str));
		buffer.append(PathExpressionUtil.SINGLE_QUOTE);
		return buffer.toString();
	}
	
	public static String escapeStringLiteral(String str) {
		if (str == null) {
			return null;
		}
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < str.length(); i++) {
			char ch = str.charAt(i);
			if (ch == '\'') {
				buffer.append(PathExpressionUtil.ESCAPED_SINGLE_QUOTE);
			} else if (ch == '\"') {
				buffer.append(PathExpressionUtil.ESCAPED_DOUBLE_QUOTES);
			} else {
				buffer.append(ch);
			}
		}
		String result = buffer.toString();
		return result;
	}
	
	public static String unescapeStringLiteral(String str) {
		if (str == null) {
			return null;
		}
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < str.length(); i++) {
			char ch = str.charAt(i);
			if (i + 1 < str.length()) {
				String next2Char = str.substring(i, i + 2);
				if (PathExpressionUtil.ESCAPED_SINGLE_QUOTE.equals(next2Char)) {
					buffer.append(PathExpressionUtil.SINGLE_QUOTE);
					i++;
				} else if (PathExpressionUtil.ESCAPED_DOUBLE_QUOTES.equals(next2Char)) {
					buffer.append(PathExpressionUtil.DOUBLE_QUOTES);
					i++;
				} else {
					buffer.append(ch);
				}
			} else {
				buffer.append(ch);
			}
		}
		String result = buffer.toString();
		return result;
	}
	
	/**
	 * Returns the closest common ancestor of both objects.
	 * Precondition: object1 and object2 must reside in the same EMF resource,
	 * thus a common ancestor must exist.
	 * @param object1
	 * @param object2
	 * @return
	 */
	static EObject getCommonAncestor(EObject object1, EObject object2) {
		if (object1 != null && object2 != null) {
			if (object1 == object2) {
				return object1;
			} else if (isAncestor(object1, object2)) {
				// Object1 is an ancestor of object2.
				return object1;
			} else if (isAncestor(object2, object1)) {
				// Object2 is an ancestor of object1.
				return object2;
			}
			
			EObject current = object1.eContainer();
			while (current != null) {
				if (isAncestor(current, object2)) {
					// If current is an ancestor of object2.
					return current;
				}
				current = current.eContainer();
			}
		}
		// Cannot find common ancestor.
		return null;
	}
	
	/**
	 * Returns true if the parent object is an ancestor of the child object.
	 * @param parent The parent object.
	 * @param child The child objcet.
	 * @return
	 */
	static boolean isAncestor(EObject parent, EObject child) {
		if (child != null) {
			EObject current = child.eContainer();
			while (current != null) {
				if (current == parent) {
					return true;
				}
				current = current.eContainer();
			}
		}
		return false;
	}
	
	/**
	 * Returns the context root for an absolute path.
	 * Note: In MME, the context root for an abosulte is always the MontiorType.
	 * @param context
	 * @return MonitorType.
	 */
	static EObject getAbsolutePathContextRoot(EObject context) {
		EObject current = context;
		while (current != null) {
			// CM: URI-to-XPath conversion... when crafting absolute paths, we need to start from DocRoot...
//			if (current instanceof MonitorType) {
//				return current;
//			} else if (current instanceof EventDefinitionListType) {
//				return current;
//			} 
			if (current instanceof DocumentRoot) {
				return current;
			} else if (current instanceof EventDefinitionListType) {
				return current;
			} 

			current = current.eContainer();
		}
		return null;
	}
	
	static EObject getEventDefinition(EObject eventAttribute) {
		EObject current = eventAttribute;
		while (current != null) {
			if (current instanceof EventDefinitionType) {
				return current;
			} 
			current = current.eContainer();
		}
		return null;
	}
	

}
