/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2006, 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
/*
 * Created on 14-Jul-2006
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package com.ibm.wbimonitor.xml.model.util;

import java.util.List;

import org.apache.commons.jxpath.JXPathContext;
import org.apache.commons.jxpath.JXPathException;
import org.apache.commons.jxpath.JXPathIntrospector;
import org.apache.commons.jxpath.Pointer;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.EObject;

import com.ibm.wbimonitor.xml.model.ModelPlugin;
import com.ibm.wbimonitor.xml.model.eventdefinition501.util.Constants;

/**
 * @author andyfu
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class PathExpressionUtil {
public static final String COPYRIGHT="(C) Copyright IBM Corporation 2006, 2007.";  //$NON-NLS-1$


	public static final String PARENT_STEP = "..";
	public static final String SELF_STEP = ".";
	public static final String STEP_SEPARATOR = "/";
	public static final String SELF_STEP_SEPARATOR = "./";
	public static final String SINGLE_QUOTE = "'";
	public static final String DOUBLE_QUOTES = "\"";
	public static final String ESCAPED_SINGLE_QUOTE = "''";
	public static final String ESCAPED_DOUBLE_QUOTES = "\"\"";
	public static final String CONTEXT_DATA_STEP = Constants.CONTEXT_DATA;
	public static final String EXTENDED_DATA_STEP = Constants.EXTENDED_DATA;
	// TODO: use constants provided by event definition
	public static final String PREDEFINED_DATA_STEP = "predefinedData"; 
	
	/**
	 * Returns an XPath 1.0 Path Expression that points to the target EObject
	 * rooted from the context EObject. 
	 * 
	 * Precondition: The context object and the target object must reside in 
	 * the same EMF Resource.
	 * 
	 * @param context The context object for the path expression.
	 * @param target The target object for the path expression.
	 * @return An XPath 1.0 path expression, null if the parameters fails to meet precondition requirements.
	 */
	public static String getPathExpression(EObject context, EObject target) {
		EObject myContext = null;
		if (context == null) {
			myContext = PathExpressionHelper.getAbsolutePathContextRoot(target);
		} else {
			myContext = context;
		}
		
		// Validate the context and target objects.
		if (!PathExpressionHelper.isValid(myContext, target)) {
			return null;
		}
		StringBuffer buffer = new StringBuffer();
		List objectList = PathExpressionHelper.collectIntermediateEObjects(myContext, target);
		for (int i = 0; i < objectList.size() - 1; i++) {
			if (i != 0 || context == null) {
				buffer.append(STEP_SEPARATOR);
			}
			EObject stepSource = (EObject) objectList.get(i);
			EObject stepTarget = (EObject) objectList.get(i + 1);
			String step = PathExpressionHelper.generatorPathExpressionStep(stepSource, stepTarget);
			if (step != null) {
				buffer.append(step);
			}
		}
		
		String result = buffer.toString();
		
		// Insert ./ at the front if the expression is a single step non-NC name expression.
		if (objectList.size() == 2 && context != null) {
			if (result.startsWith(SINGLE_QUOTE) && result.endsWith(SINGLE_QUOTE)) {
				result = SELF_STEP_SEPARATOR + result;
			}
		}
		
		return result;
	}
		
	
	/**
	 * Returns an XPath 1.0 Path Expression that points to the target Event defintiion attribute
	 * from a Monitoring model context EObject via an Monitoring model inbound/outbound event.
	 * 
	 * Precondition: The mmContext and the mmEvent must reside in 
	 * the same EMF Resource. The eventAttribute must be owned by an EventDefinitionType.
	 * 
	 * @param mmContext A monitoring model context object. if null, absoulte path.
	 * @param mmEvent A monitoring model inbound/outbound event.
	 * @param eventAttributet The target event attribute.
	 * @return An XPath 1.0 path expression, null if the parameters fails to meet precondition requirements.
	 */
	public static String getMm2CbePathExpression(EObject mmContext, EObject mmEvent, EObject eventAttribute) {
		EObject eventDefinition = PathExpressionHelper.getEventDefinition(eventAttribute);
		if (eventDefinition == null) {
			return null;
		}
		String mmPathExpression = getPathExpression(mmContext, mmEvent);
		String cbePathExpression = getPathExpression(eventDefinition, eventAttribute);
		
		if (mmPathExpression != null && cbePathExpression != null) {
			StringBuffer buffer = new StringBuffer();
			buffer.append(mmPathExpression);
			buffer.append(STEP_SEPARATOR);
			buffer.append(cbePathExpression);
			return buffer.toString();
		}
		return null;
	}
	
	/**
	 * Returns true if the given xPath path expression is absolute, false otherwise.
	 * @param xPath The XPath path expression
	 * @return True if the path expression is absolute (i.e. non-relative)
	 */
	public static boolean isAbsolutePath(String xPath){
		if (xPath != null) {
			return xPath.trim().startsWith(STEP_SEPARATOR);
		} else {
			return false;
		}
	}
	
	/**
	 * Evaluates an XPath 1.0 Path expression the context root resolved by the contextResolver, given
	 * the source object.
	 * 
	 * @param contextResolver The context resolver to use
	 * @param source The source object on which the context resolver operates
	 * @param xPath The XPath 1.0 path expression to evaluate
	 * @return The result object, null if it cannot be resolved
	 */
	public static EObject evaluatePathExpression(ModelContextResolver contextResolver, EObject source, String xPath){
		if (isAbsolutePath(xPath)){
			return evaluatePathExpression(contextResolver.getAbsolutePathContextObject(source), xPath);
		} else {
			return evaluatePathExpression(contextResolver.getContextObject(source), xPath);
		}
	}
	
	/**
	 * Evaluate an XPath 1.0 Path expression against a given context root.
	 * @param context The context object to evaluate against.
	 * @param xpath An XPath 1.0 Path expression.
	 * @return The result object, null if cannot resolve.
	 */
	public static EObject evaluatePathExpression(EObject context, String xpath) {
		Object result = null;
		if (context != null && xpath != null) {
			try {
				JXPathIntrospector.registerDynamicClass(MmDynamicContextBean.class, MmDynamicPropertyHandler.class);
				JXPathContext xpathContext = buildDynamicContextBean(context);
				result = xpathContext.getValue(xpath);
			} catch (JXPathException cause) {
				// Logging is NOT necessary, okay to have invalid expressions.
			}
		}		
		if (result instanceof MmDynamicContextBean) {
			return (EObject) ((MmDynamicContextBean) result).getContextObject();
		}
		return null;
	}
	
	private static JXPathContext buildDynamicContextBean(EObject context) {
		JXPathContext result = null;
		EObject root = PathExpressionHelper.getAbsolutePathContextRoot(context);
		String path = getPathExpression(root, context);
		if (path != null) {
			JXPathContext rootJXPathContext = JXPathContext.newContext(new MmDynamicContextBean(root));
			Pointer pointer = rootJXPathContext.getPointer(path);
			result = rootJXPathContext.getRelativeContext(pointer);
		}
		

		if (result == null) {
			result = JXPathContext.newContext(new MmDynamicContextBean(context));
		}
		return result;	
	}
	
	
}
