/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2006, 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.wbimonitor.xml.model.util;

import org.eclipse.emf.ecore.EObject;

import com.ibm.wbimonitor.xml.model.mm.ActionType;
import com.ibm.wbimonitor.xml.model.mm.ActionsType;
import com.ibm.wbimonitor.xml.model.mm.MonitorType;
import com.ibm.wbimonitor.xml.model.mm.ReferenceType;
import com.ibm.wbimonitor.xml.model.mm.SetDiagramLinkType;
import com.ibm.wbimonitor.xml.model.mm.VisualModelType;
import com.ibm.wbimonitor.xml.model.mm.VisualizationType;

public class VisualModelContextResolver extends ModelContextResolver {
public static final String COPYRIGHT="(C) Copyright IBM Corporation 2006, 2007.";  //$NON-NLS-1$

	
	private static VisualModelContextResolver sInstance;
	
	private VisualModelContextResolver(){
	}
	
	public static VisualModelContextResolver getInstance(){
		if (sInstance == null){
			sInstance = new VisualModelContextResolver();
		}
		return sInstance;
	}
	
	public EObject getContextObject(EObject source, EObject target){
		
		EObject context = getSpecialContext(source);
		
		if (context == null){			
			context = super.getContextObject(source, target);
		}
		return context;
	}
	
	/**
	 * This implementation of the getContextObject() method takes advantage
	 * of the fact that all references from objects inside the VisualModel
	 * have the enclosing Visualization as their source object
	 */
	public EObject getContextObject(EObject source){
		
		EObject context = getSpecialContext(source);
		
		if (context != null){
			return context;
		}
		
		EObject parent = source;
		while (parent != null && !(parent instanceof VisualizationType)){
			parent = parent.eContainer();
		}
		
		if (parent instanceof VisualizationType){
			return (VisualizationType)parent;
		}
		
		return null;
		
	}
	
	private EObject getSpecialContext(EObject source){
		EObject context = null;
		
		if (source instanceof ReferenceType){
			source = source.eContainer();
		}
	
		
		if (source instanceof VisualizationType || source instanceof SetDiagramLinkType || source instanceof VisualModelType){
			// for VisualizationType --> ContextType, context = MonitorType, since the target could be a KPI Context
			//  or a MonitoringContext
			context = (MonitorType)((VisualModelType)getModel(source)).eContainer();	
		} else if (source instanceof ActionType){
			while(!(source instanceof VisualizationType)){
				source = source.eContainer();
			}
			context = ((VisualizationType)source).getContextObject();
		}
		
		return context;
	}			
	
}
