/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */ 
package com.ibm.wbimonitor.xml.model.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;

import com.ibm.wbimonitor.xml.model.mm.MonitorType;

/**
 * This class provides access to the JAR files that implement or support the implementation
 * of the user defined XPath functions that are accessible from a given monitor model. 
 *
 */
public class XPathJarResolver  {
    public static final String COPYRIGHT = "(C) Copyright IBM Corporation 2007.";  //$NON-NLS-1$


	/**
	 * This class returns a map of URIs to a (possibly empty) list of Java class names.  The URI represents the 
	 * location of a given JAR file and the list of class names identifies the classes within that JAR file
	 * that contain implementations of user defined XPath functions for use within the Monitor model.  All of the 
	 * JAR file URIs are within the scope of the given monitor model provided as input.
	 * 
	 * If the map value list is empty, it indicates that although there are no user defined function definitions
	 * within the given JAR file, the user has indicated that the JAR file is required on the classpath
	 * of other JAR files identified in the Map for purposes of execution.
	 * 
	 * @param monitor The monitor model that defines the JAR file search scope
	 * @return A map of JAR file URIs to the (possibly empty) list of java class names that implement 
	 * user defined XPath functions for use in a monitor model.
	 */
	public static Map<URI, List<String>> resolveJars(MonitorType monitor){
		Map<URI, List<String>> map = new HashMap<URI, List<String>>();
		IConfigurationElement[] locationResolvers = Platform.getExtensionRegistry().getConfigurationElementsFor("com.ibm.wbimonitor.xml.model.xpathFunctionLocationResolver");
		for(int i = 0; i < locationResolvers.length; i++){
			IXPathFunctionLocationResolver resolver = null;
			try {
				resolver = (IXPathFunctionLocationResolver)locationResolvers[i].createExecutableExtension("class");
				map = resolver.resolveJars(monitor);
				if(map != null && !map.isEmpty()){
					break;
				}
			} 
			catch (CoreException e){
				Logger.log(IStatus.ERROR, "Failed to create XPath function location resolver", e);
				continue;
			}
		}
		return map;
	}
}
