/* IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * (C) Copyright IBM Corporation 2006, 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.wbimonitor.xml.model.xpath;

/*******************************************************************************
 * 
 * Implementation class for nodes in an abstract syntax tree.
 *  
 ******************************************************************************/

public class SimpleNode implements Node {
public static final String COPYRIGHT="(C) Copyright IBM Corporation 2006, 2007.";  //$NON-NLS-1$


    /** The node value */
    String m_value;
    
    /** XPath parser */
    XPath theParser;

    /** The parentNode of this node. */
    Node parentNode;

    /** The children of this node. */
    Node[] children;

    /**
     * The id of this node, as defined in XPathTreeConstants.java (indicating
     * the node type)
     */
    public int id;

    /** Constructor initializing the node id (node type) and setting a parser */
    public SimpleNode(XPath aParser, int newId) {
        theParser = aParser;
        this.id = newId;
    }

    /** Constructor initializing the node id (node type) */
    public SimpleNode(int newId) {
        this.id = newId;
    }

    /** Implementation of jjtCreate() */
    public static Node jjtCreate(XPath p, int id) {
        return new SimpleNode(p, id);
    }

    /** Set parentNode from argument. */
    public void jjtSetParent(Node node) {
        this.parentNode = node;
    }

    /** Get parentNode node. */
    public Node jjtGetParent() {
        return this.parentNode;
    }

    /** Dummy-implementation for jjtOpen() */
    public void jjtOpen() {        
        beginLn = theParser.token.beginLine;
        beginCol = theParser.token.beginColumn;
    }

    /** Dummy-implementation for jjtClose() */
    public void jjtClose() {        
        endLn = theParser.token.endLine;
        endCol = theParser.token.endColumn;
    }

    /** Implementation of jjtAddChild() */
    public void jjtAddChild(Node node, int position) {

        SimpleNode childNode = (SimpleNode) node;
        if (childNode.id == XPathTreeConstants.JJTABBREVREVERSESTEP
                || childNode.id == XPathTreeConstants.JJTCONTEXTITEMEXPR) {
            childNode.processToken(childNode.theParser.token);
        }

        if (this.id == XPathTreeConstants.JJTNCNAME && childNode.id == XPathTreeConstants.JJTQNAME) {
            this.m_value = childNode.m_value;
            return;
        }

        else if (this.id == XPathTreeConstants.JJTQNAME && childNode.id == XPathTreeConstants.JJTFUNCTIONQNAME) {
            this.m_value = childNode.m_value;
            this.beginCol = childNode.beginCol;
            this.beginLn = childNode.beginLn;
            this.endCol = childNode.endCol;
            this.endLn = childNode.endLn;
            return;
        }
        
        if (this.children == null) {
            this.children = new Node[position + 1];
        } else if (position >= this.children.length) {
            Node newChildren[] = new Node[position + 1];
            for (int j=0; j< this.children.length; j++) newChildren[j] = this.children[j];
            this.children = newChildren;
        }
        this.children[position] = node;
    }

    /** Implementation of jjtGetChild() */
    public Node jjtGetChild(int position) {
        return children[position];
    }

    /** Implementation of jjtGetNumChildren() */
    public int jjtGetNumChildren() {
        return (children == null) ? 0 : children.length;
    }

    /** Accept a visitor for this node. */
    public Object jjtAccept(XPathVisitor visitor, Object userData) {
        return visitor.visit(this, userData);
    }

    /** Have all children accept a visitor. Note that any value that the children might return from visit() is lost. */
    public Object childrenAccept(XPathVisitor visitor, Object userData) {
        if (children != null) {
            for (int i = 0; i < children.length; ++i) {
                children[i].jjtAccept(visitor, userData);
            }
        }
        return userData;
    }

    /** Initialize instance data from token */
    public void processToken(Token token) {
        this.m_value = token.image;
        this.beginCol = token.beginColumn;
        this.endCol = token.endColumn;
        this.beginLn = token.beginLine;
        this.endLn = token.endLine;
    }

    /** Return node type (as a string) */
    public String toString() {
        return XPathTreeConstants.jjtNodeName[this.id];
    }
    
    /** Return node type (as a string) prefixed by the indentation passed as an argument */
    public String toString(String indent) {
        return indent + XPathTreeConstants.jjtNodeName[this.id] + " start=" + this.beginCol + " end=" + this.endCol;
    }

    /** Perform dump to System.out */
    public void dump(String prefix) {
        this.dump(prefix, System.out);
    }

    /** Perform dump to stream passed as second argument: print node type and node value, followed by dump of children */
    public void dump(String prefix, java.io.PrintStream stream) {
        stream.print(toString(prefix));
        this.printValue(stream);
        stream.println();
        if (children != null) {
            for (int i = 0; i < children.length; ++i) {
                SimpleNode child = (SimpleNode) children[i];
                if (child != null) child.dump(prefix + "   ", stream);
            }
        }
    }

    /** Get node value */
    public String getValue() { return this.m_value; }

    /** Set node value */
    public void setValue(String nodeValue) { this.processValue(nodeValue); }

    /** Set node value (alternative) */
    public void processValue(String newNodeValue) { this.m_value = newNodeValue; }

    /** Print node value, prefixed by a blank */
    public void printValue(java.io.PrintStream ps) {
        ps.print((this.m_value == null) ? "" : (" " + this.m_value));
    }

    /** Get user data */
    public Object getUserValue() { return this.userData; }

    /** Set user data */
    public void setUserValue(Object data) { this.userData = data; }
    
    public int beginCol = -1;

    public int endCol = -1;

    public int beginLn = -1;

    public int endLn = -1;
    
    private Object userData;

}
