# ************************************************************************
#  Licensed Material - Property of IBM
# 
#  5724-M24 
# 
#  (c) Copyright IBM Corp. 2006, 2007.  All rights reserved.
# 
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# ************************************************************************
# Created: 07/12/2007

import sys, os,re
from os import environ
from java.io import File
from java.lang import System
from java.text import SimpleDateFormat 
from java.util import Date
from ivtout import outputResult
from win.path import removePathSpace 
from com.ibm.wbmonitor.install.common.log import ILogger

__all__ = []

profDir = ""
logger  = None

className = "ivu"


def init(profileDir, log):
  global profDir, logger
  profDir = profileDir
  logger  = log


def ivuOnComp(components, ivuLogFileName):
  global profDir, logger
  methodName = "ivuOnComp"
  errorCount = 0
  ivuLog = ivuLogFileName 

  import tempfile #global import doesn't work here



  # Remove space in PATH on Windows
  if (System.getProperty("os.name").startswith("Windows")):
    winProfileDir = removePathSpace(profDir)
    ivuLog = removePathSpace(ivuLog)

  #IVU log file 
  #date = Date()
  #f = SimpleDateFormat("yyyy_MM_dd_HHmmss")
  #ivuLogFileName = profDir + File.separator + "logs" \
  #   + File.separator + "ivu_" + f.format(date) + ".log"

  resultPattern = re.compile('([\d]+)$')
  for comp in components:
    
    tmpFile =  tempfile.mktemp() 
    if (System.getProperty("os.name").startswith("Windows")):
      cmd = os.path.join(winProfileDir, "bin")
      cmd = os.path.join(cmd, "installver")
      cmd = cmd + ".bat"
      cmd = cmd + " -includecomponent " + comp + " -userpropertyfile ivu_wbm.template -log " +  \
            tmpFile + " > NUL" 

      #If remove Windows path space doesn't work need wrapper a tmp batch file
      #out = open(r'ivttmp.bat', 'w')
      #out.write("call " + cmd)
      #out.close()
      #cmd = r'ivttmp.bat >NUL'
    else:
      cmd = os.path.join(profDir, "bin")
      cmd = os.path.join(cmd, "installver")
      cmd = cmd + ".sh"
      cmd = cmd + " -includecomponent " + comp + " -userpropertyfile ivu_wbm.template -log " +  \
            tmpFile + " > /dev/null" 

    #print cmd
    os.system(cmd)
    logger.msg(ILogger.TYPE_INFO, className, methodName, cmd)



    if (not os.path.exists(tmpFile)):
      mesg = "Can't run IVU test from firststep. You can run <WAS_HOME>/firststeps.wbm/ivt/"
      #It probably crash on AIX if run IVT from firststep browser
      if (System.getProperty("os.name").startswith("Windows")):
        mesg += "ivu.bat"
      else:
        mesg += "ivu.sh"
      outputResult("item", "IVU test", {'status':'failed'})
      logger.msg(ILogger.TYPE_WARN, className, methodName, mesg)
      errorCount += 1
      output = open(ivuLog, 'a')
      output.write(mesg)
      output.write("manually.")
      output.close()
      break


    #output = open(ivuLog, 'a')
    #for line in lines:
    #  output.write(line)
    #output.close()

    if (System.getProperty("os.name").startswith("Windows")):
      cmd = "more " + tmpFile + " >> " + ivuLog
    else:
      cmd = "cat " + tmpFile + " >> " + ivuLog
    os.system(cmd)

    input = open(tmpFile, 'r')
    lines = input.readlines()
    input.close()
    os.remove(tmpFile)


    #process output
    listSize = len(lines)
    #print lines[listSize-2]
    summaryLine = lines[listSize-2]
    summaryLine = summaryLine.strip()
    summaryLineItems = summaryLine.split(' ')
    if (summaryLineItems[1] != "CWNVU0400I:"):
      mesg = comp + ": no such component "
      outputResult("item", comp, {'status':'failed'})
      logger.msg(ILogger.TYPE_WARN, className, methodName, mesg)
      errorCount += 1
      continue
    m = resultPattern.search(summaryLine)
    numError = 0
    if m:
      numError = int(m.group(1))
    #numError = int(summaryLineItems[len(summaryLineItems)-1])
    if (numError == 0):
      mesg = comp + " passed"
      outputResult("item", comp, {'status':'passed'})
      logger.msg(ILogger.TYPE_INFO, className, methodName, mesg)
    else:
      if (numError > 1):
        mesg = comp + " failed with " + str(numError) + " errors"
      else:
        mesg = comp + " failed with " + str(numError) + " error"
      outputResult("item", mesg, {'status':'failed'})
      logger.msg(ILogger.TYPE_WARN, className, methodName, mesg)
      errorCount += numError

  return errorCount

