#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2008. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#

from java.lang import System
import sys
import re
import sre

#------------------------------------------------------------------------------
# a utility method
#------------------------------------------------------------------------------

def wsadminToList(inStr):
  outList=[]
  if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
    inStr = inStr[1:-1]
    tmpList = inStr.split(" ")
  else:
    tmpList = inStr.split("\n")  #splits for Windows or Linux
  for item in tmpList:
    item = item.rstrip();        #removes any Windows "\r"
    if (len(item)>0):
      outList.append(item)
  return outList
#endDef

#------------------------------------------------------------------------------
# check whether the specified application is installed on specified cell
#------------------------------------------------------------------------------
def checkApplicationExist(applicationName):
    exist = 0
    applications = wsadminToList(AdminApp.list())
    for application in applications:
        if (application == applicationName):
            exist = 1
            return exist
    return exist

#------------------------------------------------------------------------------
# get the property value by the the property name; If property not available, return ''
#------------------------------------------------------------------------------
def getPropertyValue(propertyName):
    propertyValue = java.lang.System.getProperty(propertyName)
    if(propertyValue == None):
        return ''
    return propertyValue


def getWASHome():
    return getPropertyValue('was.install.root')

def getCellName():
    return getPropertyValue('local.cell')

def getUpdatedNode():
    return getPropertyValue('local.node')

#------------------------------------------------------------------------------
# get the servers defined in specified cell and node; need to filter out the node agent, cluster member
#------------------------------------------------------------------------------
def getServerList(cell,node):
    serverNameList = []
    nodeId = AdminConfig.getid('/Cell:' + cell + '/Node:' + node)
    if(nodeId == ''):
        return serverNameList
    serverList = wsadminToList(AdminConfig.list('Server',nodeId))
    for server in serverList:
        serverName = AdminConfig.showAttribute(server,'name')
        clusterName = AdminConfig.showAttribute(server,'clusterName')
        if(clusterName == None):
            if(serverName != 'nodeagent'):
                serverNameList.append(serverName)
    return serverNameList

#------------------------------------------------------------------------------
# check whether the specified application is installed on specified cell
#------------------------------------------------------------------------------
def checkApplicationExist(applicationName):
    exist = 0
    applications = wsadminToList(AdminApp.list())
    for application in applications:
        if (application == applicationName):
            exist = 1
            return exist
    return exist

#------------------------------------------------------------------------------
# install the application with the specified parameter
#------------------------------------------------------------------------------
def updateApplication(applicationName, earLocation, cell, node, server):
    result = 0
    whetherApplicationExist = checkApplicationExist(applicationName)
    
    print "applicationName="+applicationName+" earLocation="+earLocation+" cell="+cell+" node="+node+" server="+server
    
    if(whetherApplicationExist == 0):
        print applicationName+ " does not exist. Do nothing."
    else:
        print "Updating "+applicationName
        try:
            AdminApp.update(applicationName,"app", ["-operation", "update", "-contents",earLocation])
            AdminConfig.save()  
        except:
            print "Exception occurs in AdminApp.update for "+applicationName    
            result = -1
        
    return result;

                                                 
washome =  getWASHome()
cell = getCellName()
node = getUpdatedNode()
serverList = getServerList(cell,node)
appLocationPrefix = washome+"/installableApps.wbm/";

rc = 0
for server in serverList:
    rc += updateApplication("IBM_WBM_DMS_SERVICE",appLocationPrefix+"DmsService.ear",cell,node,server)
    rc += updateApplication("IBM_WBM_ACTIONSERVICES",appLocationPrefix+"monactionmgr.ear",cell,node,server)
    rc += updateApplication("IBM_WBM_REST_SERVICES",appLocationPrefix+"MonitorRestServices.ear",cell,node,server)
    rc += updateApplication("IBM_WBM_WEB_DASHBOARD",appLocationPrefix+"WBMDashboardWeb.ear",cell,node,server)
    rc += updateApplication("IBM_WBM_ABX_WEB_DASHBOARD",appLocationPrefix+"WBMDashboardABX.ear",cell,node,server)
    

if(rc != 0):
    raise ValueError("Updating application failed.")
