-- Licensed Materials - Property of IBM
-- 5724-M24
-- c Copyright IBM Corporation 2007
-- The source code for this program is not published or otherwise
-- divested of its trade secrets, irrespective of what has been
-- deposited with the U.S. Copyright Office.
-- ------------------------------------------------------------------------------------
--
-- Scriptfile to create database, tablespaces, and schema (tables) for DB2 z/OS v8
--
---------------------------------------------------------------------------------------
-- Following variables need to be changed for customization before running this script: 
-- $STOGRP$ = DB2 Storage group name (default is SYSDEFLT)
-- $DBNAME$ = the database name, eg MONITOR
-- $SCHEMA$ = the schema name, eg MONITOR
-- Note: Tablespaces should be created with LOCKSIZE ROW to avoid deadlocks
--       Tablespaces should be created with CCSID UNICODE for
--       performance reasons and codepage coverage for user input
--
-- Run the script in the DB2 command line processor.
--
-- USAGE:   db2 -tf createDatabaseDb2Zos.ddl
--
-- ------------------------------------------------------------------------------------
-- create the database
CREATE DATABASE $DBNAME$ CCSID UNICODE;

------------- bufferpool for regular tablespaces
-- created by DBA as BP32K

------------- bufferpool for temp tablespace
-- created by DBA as TMPBP32K

------------- tablespace for DataServices (DS) 
CREATE TABLESPACE DSTS32K  IN $DBNAME$ USING STOGROUP $STOGRP$ BUFFERPOOL BP32K LOCKSIZE ROW CCSID UNICODE;

------------- tablespace for Data Mart Services (DMS)
CREATE TABLESPACE DMSTS32K IN $DBNAME$ USING STOGROUP $STOGRP$ BUFFERPOOL BP32K LOCKSIZE ROW CCSID UNICODE;

------------- tablespace (temp) for sorts/joins
-- created by DBA with DB2 CREATE TABLESPACE TMPTS32K IN TMPDB BUFFERPOOL TMPBP32K

--------------------------------------------------
-- Create auxiliary tablespaces for LOB columns --
--------------------------------------------------
CREATE LOB TABLESPACE DSLOB01 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB02 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB03 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB04 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB05 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB06 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB07 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB08 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB09 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB0A IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB0B IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB0C IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB0D IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB0E IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB0F IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB10 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB11 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB12 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB13 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB14 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB15 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB16 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB17 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB18 IN $DBNAME$ USING STOGROUP $STOGRP$;
CREATE LOB TABLESPACE DSLOB19 IN $DBNAME$ USING STOGROUP $STOGRP$;

-------------------
-- Create tables --
-------------------

CREATE TABLE $SCHEMA$.SCHEMA_VERSION
(
  SCHEMA_VERSION INTEGER NOT NULL ,
  DATA_MIGRATION SMALLINT NOT NULL ,
  PRIMARY KEY (SCHEMA_VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.SCHEMA_P ON
  $SCHEMA$.SCHEMA_VERSION (SCHEMA_VERSION) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.UNIQUE_KEY_T
(
  NEXT_KEY INTEGER NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (NEXT_KEY)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.UNIQUE_KEY_P ON
  $SCHEMA$.UNIQUE_KEY_T (NEXT_KEY) USING STOGROUP $STOGRP$;


CREATE TABLE $SCHEMA$.META_MODEL_UNVERSIONED_T
(
  MODEL_ID VARCHAR(128) NOT NULL ,
  GENERAL_PROPERTIES BLOB(1000M),
  INTEGRATION_STATUS VARCHAR(32),
  INTEGRATION_TYPE VARCHAR(32),
  CEICELL VARCHAR(128),
  CEINODE VARCHAR(128),
  CEISERVER VARCHAR(128),
  CEICLUSTER VARCHAR(128),
  CEIHOST VARCHAR(128),
  CEIPORT VARCHAR(128),
  CEIUSER_ID VARCHAR(128),
  CEIPASSWORD VARCHAR(128),
  CEIEVENT_GROUP_PROFILE_LIST VARCHAR(128),
  MEGA_NAME_MAPPER BLOB(1000M),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (MODEL_ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_MODEL_UNVERSIONED_P ON
  $SCHEMA$.META_MODEL_UNVERSIONED_T (MODEL_ID) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column GENERAL_PROPERTIES
CREATE AUX TABLE $SCHEMA$.AT25A01
   IN $DBNAME$.DSLOB01
   STORES $SCHEMA$.META_MODEL_UNVERSIONED_T COLUMN GENERAL_PROPERTIES;
-- auxiliary index for LOB column GENERAL_PROPERTIES
CREATE UNIQUE INDEX $SCHEMA$.AT25A01I
  ON $SCHEMA$.AT25A01 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column MEGA_NAME_MAPPER
CREATE AUX TABLE $SCHEMA$.AT25A16
   IN $DBNAME$.DSLOB16
   STORES $SCHEMA$.META_MODEL_UNVERSIONED_T COLUMN MEGA_NAME_MAPPER;
-- auxiliary index for LOB column MEGA_NAME_MAPPER
CREATE UNIQUE INDEX $SCHEMA$.AT25A16I
  ON $SCHEMA$.AT25A16 USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_MODEL_T
(
  MODEL_ID VARCHAR(128) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  SCHEMA_NAME VARCHAR(128),
  DISPLAY_NAME VARCHAR(1000),
  CURRENT_STATE VARCHAR(32),
  INITIAL_STATE VARCHAR(32),
  TARGET_STATE VARCHAR(32),
  APPLICATION VARCHAR(64),
  MODEL BLOB(1000M),
  CUBES BLOB(1000M),
  DIAGRAMS BLOB(1000M),
  SCHEMA_DDLS BLOB(1000M),
  DMS_DDLS BLOB(1000M),
  GENERAL_PROPERTIES BLOB(1000M),
  DEPLOYMENT_OPTIONS BLOB(1000M),
  NAME_MAPPER BLOB(1000M),
  UDF_JARS BLOB(1000M),
  ABXHOST VARCHAR(128),
  ABXPORT INTEGER,
  ABXUSER_ID VARCHAR(128),
  ABXPASSWORD VARCHAR(128),
  INTEGRATION_STATUS VARCHAR(32),
  REPLACEMENT_VERSION_DATE DECIMAL(19),
  IS_STOP_REQUESTED SMALLINT,
  APPLICATION_UPDATE SMALLINT,
  KPI_CACHE_REFRESH_INTERVAL SMALLINT,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (MODEL_ID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX  $SCHEMA$.META_MODEL_P ON
  $SCHEMA$.META_MODEL_T (MODEL_ID, VERSION) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column MODEL
CREATE AUX TABLE $SCHEMA$.AT25A02
   IN $DBNAME$.DSLOB02
   STORES $SCHEMA$.META_MODEL_T COLUMN MODEL;
-- auxiliary index for LOB column MODEL
CREATE UNIQUE INDEX $SCHEMA$.AT25A02I
  ON $SCHEMA$.AT25A02 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column CUBES
CREATE AUX TABLE $SCHEMA$.AT25A03
   IN $DBNAME$.DSLOB03
   STORES $SCHEMA$.META_MODEL_T COLUMN CUBES;
-- auxiliary index for LOB column CUBES
CREATE UNIQUE INDEX $SCHEMA$.AT25A03I
  ON $SCHEMA$.AT25A03 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column DIAGRAMS
CREATE AUX TABLE $SCHEMA$.AT25A04
   IN $DBNAME$.DSLOB04
   STORES $SCHEMA$.META_MODEL_T COLUMN DIAGRAMS;
-- auxiliary index for LOB column DIAGRAMS
CREATE UNIQUE INDEX $SCHEMA$.AT25A04I
  ON $SCHEMA$.AT25A04 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column SCHEMA_DDLS
CREATE AUX TABLE $SCHEMA$.AT25A05
   IN $DBNAME$.DSLOB05
   STORES $SCHEMA$.META_MODEL_T COLUMN SCHEMA_DDLS;
-- auxiliary index for LOB column SCHEMA_DDLS
CREATE UNIQUE INDEX $SCHEMA$.AT25A05I
  ON $SCHEMA$.AT25A05 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column DMS_DDLS
CREATE AUX TABLE $SCHEMA$.AT25A06
   IN $DBNAME$.DSLOB06
   STORES $SCHEMA$.META_MODEL_T COLUMN DMS_DDLS;
-- auxiliary index for LOB column DMS_DDLS
CREATE UNIQUE INDEX $SCHEMA$.AT25A06I
  ON $SCHEMA$.AT25A06 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column GENERAL_PROPERTIES
CREATE AUX TABLE $SCHEMA$.AT25A07
   IN $DBNAME$.DSLOB07
   STORES $SCHEMA$.META_MODEL_T COLUMN GENERAL_PROPERTIES;
-- auxiliary index for LOB column GENERAL_PROPERTIES
CREATE UNIQUE INDEX $SCHEMA$.AT25A07I
  ON $SCHEMA$.AT25A07 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column DEPLOYMENT_OPTIONS
CREATE AUX TABLE $SCHEMA$.AT25A08
   IN $DBNAME$.DSLOB08
   STORES $SCHEMA$.META_MODEL_T COLUMN DEPLOYMENT_OPTIONS;
-- auxiliary index for LOB column DEPLOYMENT_OPTIONS
CREATE UNIQUE INDEX $SCHEMA$.AT25A08I
  ON $SCHEMA$.AT25A08 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column NAME_MAPPER
CREATE AUX TABLE $SCHEMA$.AT25A09
   IN $DBNAME$.DSLOB09
   STORES $SCHEMA$.META_MODEL_T COLUMN NAME_MAPPER;
-- auxiliary index for LOB column NAME_MAPPER
CREATE UNIQUE INDEX $SCHEMA$.AT25A09I
  ON $SCHEMA$.AT25A09 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column UDF_JARS
CREATE AUX TABLE $SCHEMA$.AT25A0A
   IN $DBNAME$.DSLOB0A
   STORES $SCHEMA$.META_MODEL_T COLUMN UDF_JARS;
-- auxiliary index for LOB column UDF_JARS
CREATE UNIQUE INDEX $SCHEMA$.AT25A0AI
  ON $SCHEMA$.AT25A0A USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_MODEL_STEP_T
(
  MODEL_ID VARCHAR(128) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  STEP_NAME VARCHAR(32) NOT NULL ,
  LAST_SUCCESS_RUN_TIME TIMESTAMP,
  LAST_FAIL_RUN_TIME TIMESTAMP,
  IS_COMPLETE SMALLINT,
  MONITOR_VERSION VARCHAR(32),
  MESSAGE BLOB(1000M),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (MODEL_ID, VERSION, STEP_NAME)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_MODEL_STEP_P ON
  $SCHEMA$.META_MODEL_STEP_T (MODEL_ID, VERSION, STEP_NAME) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column MESSAGE
CREATE AUX TABLE $SCHEMA$.AT25A0B
   IN $DBNAME$.DSLOB0B
   STORES $SCHEMA$.META_MODEL_STEP_T COLUMN MESSAGE;
-- auxiliary index for LOB column MESSAGE
CREATE UNIQUE INDEX $SCHEMA$.AT25A0BI
  ON $SCHEMA$.AT25A0B USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_MONITOR_CONTEXT_T
(
  MCID VARCHAR(256) NOT NULL ,
  MODEL_ID VARCHAR(128) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  MCPARENT_ID VARCHAR(256),
  DIAGRAM BLOB(1000M),
  DISPLAY_NAME VARCHAR(1000) NOT NULL ,
  CUBE_NAME VARCHAR(128) NOT NULL ,
  TABLE_NAME VARCHAR(128) NOT NULL ,
  VIEW_NAME VARCHAR(128) NOT NULL ,
  DMS_TABLE_NAME VARCHAR(128) NOT NULL ,
  DMS_CHG_LOG_TABLE VARCHAR(128) NOT NULL ,  
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (MCID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_MONITOR_CONTEXT_P ON
  $SCHEMA$.META_MONITOR_CONTEXT_T (MCID, VERSION) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column DIAGRAM
CREATE AUX TABLE $SCHEMA$.AT25A0C
   IN $DBNAME$.DSLOB0C
   STORES $SCHEMA$.META_MONITOR_CONTEXT_T COLUMN DIAGRAM;
-- auxiliary index for LOB column DIAGRAM
CREATE UNIQUE INDEX $SCHEMA$.AT25A0CI
  ON $SCHEMA$.AT25A0C USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_MONITOR_METRIC_T
(
  METRIC_ID VARCHAR(384) NOT NULL ,
  MODEL_ID VARCHAR(128) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  MCID VARCHAR(256) NOT NULL ,
  METRIC_TYPE VARCHAR(14) NOT NULL ,
  IS_SORTABLE SMALLINT,
  COLUMN_NAME VARCHAR(128) NOT NULL ,
  DISPLAY_NAME VARCHAR(1000) NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (METRIC_ID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_MONITOR_METRIC_P ON
  $SCHEMA$.META_MONITOR_METRIC_T (METRIC_ID, VERSION) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_DIMENSION_T
(
  DIMENSION_ID VARCHAR(384) NOT NULL ,
  MODEL_ID VARCHAR(128) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  MCID VARCHAR(256) NOT NULL ,
  TABLE_NAME VARCHAR(128),
  DISPLAY_NAME VARCHAR(1000) NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (DIMENSION_ID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_DIMENSION_P ON
  $SCHEMA$.META_DIMENSION_T (DIMENSION_ID, VERSION) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_DIM_ATTR_T
(
  DIM_ATTRIBUTE_ID VARCHAR(512) NOT NULL ,
  MODEL_ID VARCHAR(128) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  DIMENSION_ID VARCHAR(384) NOT NULL ,
  DIM_LEVEL INTEGER NOT NULL ,
  IS_PART_OF_DIM_KEY INTEGER NOT NULL ,
  METRIC_ID VARCHAR(384) NOT NULL ,
  DISPLAY_NAME VARCHAR(1000) NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (DIM_ATTRIBUTE_ID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_DIM_ATTR_P ON
  $SCHEMA$.META_DIM_ATTR_T (DIM_ATTRIBUTE_ID, VERSION) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_MEASURE_T
(
  MEASURE_ID VARCHAR(384) NOT NULL ,
  MODEL_ID VARCHAR(128) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  MCID VARCHAR(256) NOT NULL ,
  DISPLAY_NAME VARCHAR(1000) NOT NULL ,
  AGGREGATION_TYPE VARCHAR(14) NOT NULL ,
  TRACKING_KEY VARCHAR(256),
  MEASURE_DATA_TYPE VARCHAR(14),
  SOURCE_METRIC_ID VARCHAR(384),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (MEASURE_ID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_MEASURE_P ON
  $SCHEMA$.META_MEASURE_T (MEASURE_ID, VERSION) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_KPI_CONTEXT_T
(
  KPI_CONTEXT_ID VARCHAR(384) NOT NULL ,
  MODEL_ID VARCHAR(128) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  DISPLAY_NAME VARCHAR(1000),
  DIAGRAM BLOB(1000M),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (KPI_CONTEXT_ID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_KPI_CONTEXT_P ON
  $SCHEMA$.META_KPI_CONTEXT_T (KPI_CONTEXT_ID, VERSION) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column DIAGRAM
CREATE AUX TABLE $SCHEMA$.AT25A0D
   IN $DBNAME$.DSLOB0D
   STORES $SCHEMA$.META_KPI_CONTEXT_T COLUMN DIAGRAM;
-- auxiliary index for LOB column DIAGRAM
CREATE UNIQUE INDEX $SCHEMA$.AT25A0DI
  ON $SCHEMA$.AT25A0D USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_KPI_T
(
  KPI_ID VARCHAR(384) NOT NULL ,
  MODEL_ID VARCHAR(128) NOT NULL ,
  KPI_CONTEXT_ID VARCHAR(256),
  VERSION DECIMAL(19) NOT NULL ,
  DISPLAY_NAME VARCHAR(1000),
  DESCRIPTION VARCHAR(256),
  KPI_ORIGIN SMALLINT NOT NULL ,
  KPI_DATA_TYPE SMALLINT NOT NULL ,
  TARGET DOUBLE,
  KPI_RANGE_TYPE SMALLINT NOT NULL ,
  KPI_CALC_METHOD SMALLINT NOT NULL ,
  AGGREGATED_MEASURE_ID VARCHAR(384),
  AGGREGATED_METRIC_ID VARCHAR(384),
  AGGREGATION_FUNCTION VARCHAR(10),
  VERSION_AGGREGATION SMALLINT,
  TIME_PERIOD_METRIC_ID VARCHAR(384),
  TIME_PERIOD_METHOD SMALLINT,
  REPEATING_PERIOD_DURATION SMALLINT,
  REPEATING_PERIOD_BASIS SMALLINT,
  REPEATING_PERIOD_TIMEZONE VARCHAR(30),
  ROLLING_PERIOD_DURATION SMALLINT,
  ROLLING_PERIOD_QUANTITY INTEGER,
  FIXED_PERIOD_START TIMESTAMP,
  FIXED_PERIOD_END TIMESTAMP,
  FIXED_PERIOD_TIMEZONE VARCHAR(30),
  CALCULATED_KPI_EXPRESSION CLOB(1048576),
  USER_ID VARCHAR(256),
  VIEW_ACCESS SMALLINT NOT NULL ,
  FORMAT_DECIMAL_PRECISION SMALLINT,
  FORMAT_CURRENCY VARCHAR(3),
  FORMAT_PERCENTAGE SMALLINT NOT NULL ,
  KPI_CACHE_VALUE DOUBLE,
  KPI_LAST_CALCULATION_TIME TIMESTAMP,
  KPI_CACHE_OVERRIDE_INTERVAL SMALLINT,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (KPI_ID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_KPI_P ON
  $SCHEMA$.META_KPI_T (KPI_ID, VERSION) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column CALCULATED_KPI_EXPRESSION
CREATE AUX TABLE $SCHEMA$.AT25A0E
   IN $DBNAME$.DSLOB0E
   STORES $SCHEMA$.META_KPI_T COLUMN CALCULATED_KPI_EXPRESSION;
-- auxiliary index for LOB column CALCULATED_KPI_EXPRESSION
CREATE UNIQUE INDEX $SCHEMA$.AT25A0EI
  ON $SCHEMA$.AT25A0E USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_KPI_METRIC_FILTER_T
(
  KPI_METRIC_FILTER_ID VARCHAR(512) NOT NULL ,
  KPI_ID VARCHAR(384) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  FILTER_METRIC_ID VARCHAR(512) NOT NULL ,
  FILTER_OPERATOR SMALLINT NOT NULL ,
  FILTER_OPERATOR_CASE_SENSITIVE SMALLINT NOT NULL ,
  FILTER_VALUE CLOB(1048576),
  VALUE_INDEX INTEGER NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (KPI_METRIC_FILTER_ID, VERSION, VALUE_INDEX)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_KPI_METRIC_FILTER_P ON
  $SCHEMA$.META_KPI_METRIC_FILTER_T (KPI_METRIC_FILTER_ID, VERSION, VALUE_INDEX) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column FILTER_VALUE
CREATE AUX TABLE $SCHEMA$.AT25A0F
   IN $DBNAME$.DSLOB0F
   STORES $SCHEMA$.META_KPI_METRIC_FILTER_T COLUMN FILTER_VALUE;
-- auxiliary index for LOB column FILTER_VALUE
CREATE UNIQUE INDEX $SCHEMA$.AT25A0FI
  ON $SCHEMA$.AT25A0F USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_KPI_RANGE_T
(
  KPI_RANGE_ID VARCHAR(512) NOT NULL ,
  KPI_ID VARCHAR(384) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  DISPLAY_NAME VARCHAR(1000),
  START_VALUE DOUBLE NOT NULL ,
  END_VALUE DOUBLE NOT NULL ,
  COLOR VARCHAR(7),
  ICON VARCHAR(256),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (KPI_RANGE_ID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_KPI_RANGE_P ON
  $SCHEMA$.META_KPI_RANGE_T (KPI_RANGE_ID, VERSION) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_KPI_DEPENDENCY_T
(
  PARENT_KPI_ID VARCHAR(384) NOT NULL ,
  PARENT_KPI_VERSION DECIMAL(19) NOT NULL ,
  CHILD_KPI_ID VARCHAR(384) NOT NULL ,
  CHILD_KPI_VERSION DECIMAL(19) NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (PARENT_KPI_ID, PARENT_KPI_VERSION, CHILD_KPI_ID,
 CHILD_KPI_VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_KPI_DEPENDENCY_P ON
  $SCHEMA$.META_KPI_DEPENDENCY_T
(PARENT_KPI_ID, PARENT_KPI_VERSION, CHILD_KPI_ID, CHILD_KPI_VERSION) 
USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.KEY_HELPER_T
(
  TABLE_NAME VARCHAR(128) NOT NULL ,
  NEXT_KEY INTEGER NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (TABLE_NAME)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.KEY_HELPER_P ON
  $SCHEMA$.KEY_HELPER_T (TABLE_NAME) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.DMS_METADATA_T
(
  MODEL_ID VARCHAR(128) NOT NULL ,
  MCID VARCHAR(256) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  DMS_COPY_PROC VARCHAR(128),
  TARGET_TABLE VARCHAR(128),
  LAST_TIME_STARTED DECIMAL(19),
  LAST_TIME_CMPLTD DECIMAL(19),
  DMS_INTERVAL DECIMAL(19),
  PROGRESS_FLAG SMALLINT,
  IS_NEW_FLAG SMALLINT,
  SUSPENDED_FLAG SMALLINT,
  IS_DELETED_FLAG SMALLINT,
  INVOKE_NOW    SMALLINT,
  ROWS_COPIED INTEGER,
  ROWS_PRUNED INTEGER,
  DMS_COPY_PROC_MESG VARCHAR(4000),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (MODEL_ID, MCID, VERSION)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.DMS_METADATA_P ON
  $SCHEMA$.DMS_METADATA_T (MODEL_ID, MCID, VERSION) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.DMS_HISTORY_T
(
  MODEL_ID VARCHAR(128) NOT NULL ,
  MCID VARCHAR(256) NOT NULL ,
  VERSION DECIMAL(19) NOT NULL ,
  TIME_STARTED DECIMAL(19) NOT NULL ,
  TIME_COMPLETED DECIMAL(19),
  DMS_INTERVAL DECIMAL(19),
  ROWS_COPIED INTEGER,
  ROWS_PRUNED INTEGER,
  DMS_COPY_PROC_MESG VARCHAR(4000),
  IS_DELETED_FLAG SMALLINT,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (MODEL_ID, MCID, VERSION, TIME_STARTED)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.DMS_HISTORY_P ON
  $SCHEMA$.DMS_HISTORY_T (MODEL_ID, MCID, VERSION, TIME_STARTED) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.META_OUTBOUND_EVENT_T
(
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  EVENT_ID                           VARCHAR(384)                      NOT NULL ,
  CONTEXT_ID                         VARCHAR(256)                      NOT NULL ,
  VERSION                            DECIMAL(19)                       NOT NULL ,
  DISPLAY_NAME                       VARCHAR(1000)                              ,
  EXTENSION_NAME                     VARCHAR(1000)                              ,
  CONTEXT_TYPE                       VARCHAR(100)                               ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( EVENT_ID, VERSION )
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.META_OUTBOUND_EVENT_P ON
  $SCHEMA$.META_OUTBOUND_EVENT_T (EVENT_ID, VERSION) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.ACT_MGR_ALERTS_T
(
  ALERT_ID VARCHAR(32) NOT NULL ,
  CONTEXT_DEF VARCHAR(300),
  CONTEXT_INST_ID VARCHAR(32),
  MODEL_NAME    VARCHAR(300),
  MODEL_ID      VARCHAR(128),   
  CONTEXT_NAME  VARCHAR(300),
  CONTEXT_ID    VARCHAR(256),
  MODEL_VERSION VARCHAR(128),
  BUS_SIT_NAME VARCHAR(254) NOT NULL ,
  CBE_XML BLOB(1000M),
  ALERT_SUBJECT VARCHAR(240) NOT NULL ,
  ALERT_BODY CLOB(1048576) NOT NULL ,
  ALERT_TIMESTAMP TIMESTAMP,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ALERT_ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.ACT_MGR_ALERTS_P ON
  $SCHEMA$.ACT_MGR_ALERTS_T (ALERT_ID) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column CBE_XML
CREATE AUX TABLE $SCHEMA$.AT25A12
   IN $DBNAME$.DSLOB12
   STORES $SCHEMA$.ACT_MGR_ALERTS_T COLUMN CBE_XML;
-- auxiliary index for LOB column CBE_XML
CREATE UNIQUE INDEX $SCHEMA$.AT25A12I
  ON $SCHEMA$.AT25A12 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column ALERT_BODY
CREATE AUX TABLE $SCHEMA$.AT25A13
   IN $DBNAME$.DSLOB13
   STORES $SCHEMA$.ACT_MGR_ALERTS_T COLUMN ALERT_BODY;
-- auxiliary index for LOB column ALERT_BODY
CREATE UNIQUE INDEX $SCHEMA$.AT25A13I
  ON $SCHEMA$.AT25A13 USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.DASHBOARD_ALERT_T
(
  USER_ID VARCHAR(32) NOT NULL ,
  ALERT_ID VARCHAR(32) NOT NULL ,
  ACKNOWLEDGED SMALLINT NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (USER_ID, ALERT_ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.DASHBOARD_ALERT_P ON
  $SCHEMA$.DASHBOARD_ALERT_T (USER_ID, ALERT_ID) USING STOGROUP $STOGRP$;
CREATE INDEX $SCHEMA$.DASHBOARD_ALERT_I1 ON $SCHEMA$.DASHBOARD_ALERT_T (USER_ID);

CREATE TABLE $SCHEMA$.ACT_HNDLR_T
(
  ID VARCHAR(32) NOT NULL ,
  NAME VARCHAR(64) NOT NULL ,
  TYPE VARCHAR(64) NOT NULL ,
  INVOCATION_TYPE VARCHAR(20),
  INVOCATION_PROFILE VARCHAR(1024),
  DESCRIPTION VARCHAR(1024),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.ACT_HNDLR_P ON
  $SCHEMA$.ACT_HNDLR_T (ID) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.ACTION_HANDLER_CTRL_T
(
  ACTION_ID VARCHAR(32) NOT NULL ,
  HANDLER_ID VARCHAR(32) NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ACTION_ID, HANDLER_ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.ACTION_HANDLER_CTRL_P ON
  $SCHEMA$.ACTION_HANDLER_CTRL_T (ACTION_ID, HANDLER_ID) USING STOGROUP $STOGRP$;
CREATE INDEX $SCHEMA$.ACTN_HNDLR_CTRL_I1 ON $SCHEMA$.ACTION_HANDLER_CTRL_T (ACTION_ID);

CREATE TABLE $SCHEMA$.PLUG_SVC_T
(
  ID VARCHAR(32) NOT NULL ,
  NAME VARCHAR(64) NOT NULL ,
  SERVICE_TYPE VARCHAR(20) NOT NULL ,
  SUB_TYPE VARCHAR(20) NOT NULL ,
  ACTION_HANDLER_ID VARCHAR(32) NOT NULL ,
  DESCRIPTION VARCHAR(1024),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.PLUG_SVC_P ON
  $SCHEMA$.PLUG_SVC_T (ID) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.ACT_TMPLT_T
(
  ID VARCHAR(32) NOT NULL ,
  NAME VARCHAR(256) NOT NULL ,
  TYPE VARCHAR(20) NOT NULL ,
  CONTENT BLOB(1024K) NOT NULL ,
  DESCRIPTION VARCHAR(1024),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.ACT_TMPLT_P ON
  $SCHEMA$.ACT_TMPLT_T (ID) USING STOGROUP $STOGRP$;
CREATE INDEX $SCHEMA$.ACT_TMPLT_I1 ON $SCHEMA$.ACT_TMPLT_T (NAME);
CREATE INDEX $SCHEMA$.ACT_TMPLT_I2 ON $SCHEMA$.ACT_TMPLT_T (TYPE);

-- auxiliary table for LOB column CONTENT
CREATE AUX TABLE $SCHEMA$.AT25A14
   IN $DBNAME$.DSLOB14
   STORES $SCHEMA$.ACT_TMPLT_T COLUMN CONTENT;
-- auxiliary index for LOB column CONTENT
CREATE UNIQUE INDEX $SCHEMA$.AT25A14I
  ON $SCHEMA$.AT25A14 USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.ACTION_T
(
  ID VARCHAR(32) NOT NULL ,
  NAME VARCHAR(64) NOT NULL ,
  TYPE VARCHAR(64) NOT NULL ,
  PARAMETER_SET BLOB(1024K),
  STATE VARCHAR(20),
  ALIAS_NAME VARCHAR(64),
  TEMPLATE_ID VARCHAR(32),
  HANDLER_ID VARCHAR(32),
  DESCRIPTION VARCHAR(1024),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.ACTION_P ON
  $SCHEMA$.ACTION_T (ID) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column PARAMETER_SET
CREATE AUX TABLE $SCHEMA$.AT25A15
   IN $DBNAME$.DSLOB15
   STORES $SCHEMA$.ACTION_T COLUMN PARAMETER_SET;
-- auxiliary index for LOB column PARAMETER_SET
CREATE UNIQUE INDEX $SCHEMA$.AT25A15I
  ON $SCHEMA$.AT25A15 USING STOGROUP $STOGRP$;

ALTER TABLE $SCHEMA$.ACTION_HANDLER_CTRL_T
    ADD CONSTRAINT D_ACTION_ID FOREIGN KEY (ACTION_ID)
    REFERENCES $SCHEMA$.ACTION_T(ID) ON DELETE CASCADE;

CREATE TABLE $SCHEMA$.SUBSCRIBED_USER_T
(
  USER_ID VARCHAR(32) NOT NULL ,
  ETAACTRL_ID VARCHAR(32) NOT NULL ,
  DASHBOARD SMALLINT NOT NULL ,
  EMAIL SMALLINT NOT NULL ,
  CELL SMALLINT NOT NULL ,
  PAGER SMALLINT NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (USER_ID, ETAACTRL_ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.SUBSCRIBED_USER_P ON
  $SCHEMA$.SUBSCRIBED_USER_T (USER_ID, ETAACTRL_ID) USING STOGROUP $STOGRP$;
CREATE INDEX $SCHEMA$.SUBSCRIBED_USER_I1 ON $SCHEMA$.SUBSCRIBED_USER_T (USER_ID);
CREATE INDEX $SCHEMA$.SUBSCRIBED_USER_I2 ON $SCHEMA$.SUBSCRIBED_USER_T (ETAACTRL_ID);

CREATE TABLE $SCHEMA$.ETAMATCH_T
(
  ID VARCHAR(32) NOT NULL ,
  DETERMINER_TYPE VARCHAR(20) NOT NULL ,
  EXTENSION_NAME VARCHAR(256) NOT NULL ,
  DESCRIPTION VARCHAR(1024),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.ETAMATCH_P ON
  $SCHEMA$.ETAMATCH_T (ID) USING STOGROUP $STOGRP$;
CREATE INDEX $SCHEMA$.ETAMATCH_I1 ON $SCHEMA$.ETAMATCH_T (EXTENSION_NAME);

CREATE TABLE $SCHEMA$.ATAMATCH_T
(
  ID VARCHAR(32) NOT NULL ,
  DETERMINER_TYPE VARCHAR(20) NOT NULL ,
  PARENT_ACTION_ID VARCHAR(32) NOT NULL ,
  CHILD_ACTION_ID VARCHAR(32) NOT NULL ,
  DESCRIPTION VARCHAR(1024),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.ATAMATCH_P ON
  $SCHEMA$.ATAMATCH_T (ID) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.ETAACTRL_T
(
  ID VARCHAR(32) NOT NULL ,
  ETAMID VARCHAR(32) NOT NULL ,
  ACTION_ID VARCHAR(32) NOT NULL ,
  NAME VARCHAR(264) NOT NULL ,
  DESCRIPTION VARCHAR(1024),
  BINDING_TIMESTAMP TIMESTAMP,
  CATEGORY VARCHAR(264),
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ID)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.ETAACTRL_P ON
  $SCHEMA$.ETAACTRL_T (ID) USING STOGROUP $STOGRP$;

CREATE INDEX $SCHEMA$.ETAACTRL_I1 ON $SCHEMA$.ETAACTRL_T (ETAMID);
CREATE INDEX $SCHEMA$.ETAACTRL_I2 ON $SCHEMA$.ETAACTRL_T (ACTION_ID);
CREATE INDEX $SCHEMA$.ETAACTRL_I3 ON $SCHEMA$.ETAACTRL_T (ETAMID, ACTION_ID);


CREATE TABLE $SCHEMA$.ACTION_MANAGER_PROP_T
(
  ACTION_PROP_NAME VARCHAR(100) NOT NULL ,
  ACTION_PROP_VALUE VARCHAR(240) NOT NULL ,
  VERSION_ID SMALLINT NOT NULL ,
  PRIMARY KEY (ACTION_PROP_NAME)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.ACTION_MANAGER_PROP_P ON
  $SCHEMA$.ACTION_MANAGER_PROP_T (ACTION_PROP_NAME) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.EQ_MODEL_VERSION_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  MODEL                              VARCHAR(1000)                     NOT NULL ,
  VERSION                            DECIMAL(19)                            NOT NULL ,
  FAIL_TIME                          DECIMAL(19)                                     ,
  LAST_SUB_INST                      VARCHAR(32)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID )
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.EQ_MODEL_VERSION_P ON
  $SCHEMA$.EQ_MODEL_VERSION_T (DBID) USING STOGROUP $STOGRP$;
CREATE INDEX $SCHEMA$.EQ_MODEL_VER_I1 ON $SCHEMA$.EQ_MODEL_VERSION_T (MODEL, VERSION);

CREATE TABLE $SCHEMA$.DQ_MODEL_VERSION_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  MODEL                              VARCHAR(1000)                     NOT NULL ,
  VERSION                            DECIMAL(19)                            NOT NULL ,
  FAIL_TIME                          DECIMAL(19)                                     ,
  LAST_SUB_STATUS                    VARCHAR(128)                               ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID )
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.DQ_MODEL_VERSION_P ON
  $SCHEMA$.DQ_MODEL_VERSION_T (DBID) USING STOGROUP $STOGRP$;
CREATE INDEX $SCHEMA$.DQ_MODEL_VER_I1 ON $SCHEMA$.DQ_MODEL_VERSION_T (MODEL, VERSION);

CREATE TABLE $SCHEMA$.EQ_INSTANCE_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  MV_DBID                            VARCHAR(32)                       NOT NULL ,
  ROOT_INST_ID                       VARCHAR(1000)                     NOT NULL ,
  FAIL_TIME                          DECIMAL(19)                                     ,
  LAST_SUB_TIME                      DECIMAL(19)                                     ,
  LAST_SUB_STATUS                    INTEGER                           NOT NULL ,
  LAST_SEQUENCE_NUM                  DECIMAL(19)                       NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID ),
  FOREIGN KEY ( MV_DBID ) REFERENCES $SCHEMA$.EQ_MODEL_VERSION_T( DBID ) ON DELETE CASCADE
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.EQ_INSTANCE_P ON
  $SCHEMA$.EQ_INSTANCE_T (DBID) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.EQ_FAILED_EVENT_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  INSTANCE_DBID                      VARCHAR(32)                       NOT NULL ,
  GU_INST_ID                         VARCHAR(1000)                     NOT NULL ,
  EQ_SEQUENCE                        DECIMAL(19)                       NOT NULL ,
  EVENT_SEQUENCE                     VARCHAR(512)                               ,
  F_SUMMARY                          VARCHAR(256)                      NOT NULL ,
  F_DETAILS                          BLOB(1024K)                                ,
  F_TIME                             DECIMAL(19)                       NOT NULL ,
  CREATION_TIME                      DECIMAL(19)                                ,
  EXT_NAME                           VARCHAR(64)                       NOT NULL ,
  CONSUMPTION_TIME                   DECIMAL(19)                       NOT NULL ,
  LAST_SUB_TIME                      DECIMAL(19)                                ,
  LAST_SUB_STATUS                    INTEGER                           NOT NULL ,
  HISTORY                            BLOB(20000K)                               ,
  DATA                               BLOB(1000M)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID ),
  FOREIGN KEY ( INSTANCE_DBID ) REFERENCES $SCHEMA$.EQ_INSTANCE_T( DBID ) ON DELETE CASCADE
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.EQ_FAILED_EVENT_P ON
  $SCHEMA$.EQ_FAILED_EVENT_T (DBID) USING STOGROUP $STOGRP$;
CREATE INDEX $SCHEMA$.EQ_FAILED_EVENT_I1 ON $SCHEMA$.EQ_FAILED_EVENT_T (INSTANCE_DBID);
CREATE INDEX $SCHEMA$.EQ_FAILED_EVENT_I2 ON $SCHEMA$.EQ_FAILED_EVENT_T (INSTANCE_DBID, EQ_SEQUENCE);

-- auxiliary table for LOB column HISTORY
CREATE AUX TABLE $SCHEMA$.AT25A10
   IN $DBNAME$.DSLOB10
   STORES $SCHEMA$.EQ_FAILED_EVENT_T COLUMN HISTORY;
-- auxiliary index for LOB column HISTORY
CREATE UNIQUE INDEX $SCHEMA$.AT25A10I
  ON $SCHEMA$.AT25A10 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column DATA
CREATE AUX TABLE $SCHEMA$.AT25A11
   IN $DBNAME$.DSLOB11
   STORES $SCHEMA$.EQ_FAILED_EVENT_T COLUMN DATA;
-- auxiliary index for LOB column DATA
CREATE UNIQUE INDEX $SCHEMA$.AT25A11I
  ON $SCHEMA$.AT25A11 USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.DQ_DEAD_EVENT_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  MV_DBID                            VARCHAR(32)                       NOT NULL ,
  F_SUMMARY                          VARCHAR(256)                      NOT NULL ,
  F_DETAILS                          BLOB(1024K)                                ,
  F_TIME                             DECIMAL(19)                            NOT NULL ,
  DATA                               BLOB(1000M)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID ),
  FOREIGN KEY ( MV_DBID ) REFERENCES $SCHEMA$.DQ_MODEL_VERSION_T( DBID ) ON DELETE CASCADE
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.DQ_DEAD_EVENT_P ON
  $SCHEMA$.DQ_DEAD_EVENT_T (DBID) USING STOGROUP $STOGRP$;
CREATE INDEX $SCHEMA$.DQ_DEAD_EVENT_I1 ON $SCHEMA$.DQ_DEAD_EVENT_T (MV_DBID);

-- auxiliary table for LOB column F_DETAILS
CREATE AUX TABLE $SCHEMA$.AT25A18
   IN $DBNAME$.DSLOB18
   STORES $SCHEMA$.DQ_DEAD_EVENT_T COLUMN F_DETAILS;
-- auxiliary index for LOB column F_DETAILS
CREATE UNIQUE INDEX $SCHEMA$.AT25A18I
  ON $SCHEMA$.AT25A18 USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column DATA
CREATE AUX TABLE $SCHEMA$.AT25A19
   IN $DBNAME$.DSLOB19
   STORES $SCHEMA$.DQ_DEAD_EVENT_T COLUMN DATA;
-- auxiliary index for LOB column DATA
CREATE UNIQUE INDEX $SCHEMA$.AT25A19I
  ON $SCHEMA$.AT25A19 USING STOGROUP $STOGRP$;
  
--
-- dim time
--

CREATE TABLE $SCHEMA$.DIM_TIME
(
  SURROGATE_KEY DATE NOT NULL ,
  DAY_NUM SMALLINT NOT NULL ,
  MONTH_NUM SMALLINT NOT NULL ,
  YEAR_NUM INTEGER NOT NULL ,
  PRIMARY KEY (SURROGATE_KEY)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.DIM_P ON
  $SCHEMA$.DIM_TIME (SURROGATE_KEY) USING STOGROUP $STOGRP$;

CREATE UNIQUE INDEX $SCHEMA$.IXDTIME ON
  $SCHEMA$.DIM_TIME (YEAR_NUM ASC, MONTH_NUM ASC, DAY_NUM ASC) USING STOGROUP $STOGRP$;

insert into $SCHEMA$.dim_time( surrogate_key, year_num, month_num, day_num) 
with WBITIME (skey, ldate) as
(
   select 1 as skey, DATE('2000-01-01') as ldate
   from sysibm.sysdummy1

   UNION ALL
   
   SELECT parent.skey+1, ldate + 1 DAYS
   from WBITIME parent
   where YEAR(ldate + 1 days) < 2015
)
select a.ldate, year(a.ldate), month(a.ldate), day(a.ldate)
from WBITIME a;

insert into $SCHEMA$.dim_time(surrogate_key,year_num,month_num,day_num) values (DATE('9999-01-01'),9999,1,1);

--
-- End dim time
--

-- set schema version to: 610
INSERT INTO $SCHEMA$.SCHEMA_VERSION (SCHEMA_VERSION, DATA_MIGRATION) VALUES( 610, 0 );

-- View: ActTmplt
CREATE VIEW $SCHEMA$.ACT_TMPLT(ID, NAME, TYPE, DESCRIPTION ) AS 
 SELECT $SCHEMA$.ACT_TMPLT_T.ID, $SCHEMA$.ACT_TMPLT_T.NAME, $SCHEMA$.ACT_TMPLT_T.TYPE, $SCHEMA$.ACT_TMPLT_T.DESCRIPTION 
 FROM $SCHEMA$.ACT_TMPLT_T;

-- View: ETAMatch
CREATE VIEW $SCHEMA$.ETAMATCH(ID, DETERMINER_TYPE, EXTENSION_NAME, DESCRIPTION ) AS 
 SELECT $SCHEMA$.ETAMATCH_T.ID, $SCHEMA$.ETAMATCH_T.DETERMINER_TYPE, $SCHEMA$.ETAMATCH_T.EXTENSION_NAME, $SCHEMA$.ETAMATCH_T.DESCRIPTION 
 FROM $SCHEMA$.ETAMATCH_T;

-- View: Action
CREATE VIEW $SCHEMA$.ACTION(ID, NAME, TYPE ) AS 
 SELECT $SCHEMA$.ACTION_T.ID, $SCHEMA$.ACTION_T.NAME, $SCHEMA$.ACTION_T.TYPE
 FROM $SCHEMA$.ACTION_T;

-- View: ActMgrProp
CREATE VIEW $SCHEMA$.ACT_MGR_PROP(ACTION_PROP_NAME, ACTION_PROP_VALUE ) AS 
 SELECT $SCHEMA$.ACTION_MANAGER_PROP_T.ACTION_PROP_NAME, $SCHEMA$.ACTION_MANAGER_PROP_T.ACTION_PROP_VALUE 
 FROM $SCHEMA$.ACTION_MANAGER_PROP_T;

-- View: PlugSvc
CREATE VIEW $SCHEMA$.PLUG_SVC(ID, NAME, SERVICE_TYPE, SUB_TYPE, ACTION_HANDLER_ID ) AS 
 SELECT $SCHEMA$.PLUG_SVC_T.ID, $SCHEMA$.PLUG_SVC_T.NAME, $SCHEMA$.PLUG_SVC_T.SERVICE_TYPE, $SCHEMA$.PLUG_SVC_T.SUB_TYPE, 
 $SCHEMA$.PLUG_SVC_T.ACTION_HANDLER_ID FROM $SCHEMA$.PLUG_SVC_T; 

-- View: Alerts
CREATE VIEW $SCHEMA$.ALERTS(ALERT_ID, BUS_SIT_NAME, ALERT_SUBJECT, ALERT_TIMESTAMP ) AS
 SELECT $SCHEMA$.ACT_MGR_ALERTS_T.ALERT_ID, $SCHEMA$.ACT_MGR_ALERTS_T.BUS_SIT_NAME, $SCHEMA$.ACT_MGR_ALERTS_T.ALERT_SUBJECT, $SCHEMA$.ACT_MGR_ALERTS_T.ALERT_TIMESTAMP
 FROM $SCHEMA$.ACT_MGR_ALERTS_T;

-- View:  EQs
CREATE VIEW $SCHEMA$.EQ_FE_IDS(DBID, INSTANCE_DBID, EQ_SEQUENCE) AS
 SELECT DBID, INSTANCE_DBID, EQ_SEQUENCE 
 FROM $SCHEMA$.EQ_FAILED_EVENT_T;

--
-- Populate Action Mgr Tables
--
INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Host','localhost', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Port','25', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Sender','default@127.0.0.1', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.UID.Field','uid', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.Email.Field','mail', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.CellEmail.Field','mobile', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.PagerEmail.Field','pager', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.CEI_Logging','no', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.URL','ldap://localhost:389', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.UserName','cn=root', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.Password','password', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Hostname','localhost', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Port','1414', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Channel','FMCQM.CL.TCP', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.UID.Field','uid', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Email.Field','mail', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.CellEmail.Field','mobile', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.PagerEmail.Field','pager', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Provider','$VMM_HOST$:$VMM_PORT$', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Entity.Type','PersonAccount', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID)    
  VALUES ('ActionManager.VMM.Group.Type','Group', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1001','EmailHandler','AAM_6.0_EMAIL_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.EmailActionHandler','Email Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1002','AlertHandler','AAM_6.0_ALERT_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.AlertActionHandler','Alert Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1003','WebServiceHandler','AAM_6.0_WEBSERVICES_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.WebServicesHandler','WebService Action Service Handler', 0);

--To reactivate MQWF handlers uncomment these lines
--INSERT INTO $SCHEMA$.ACT_HNDLR_T
--(ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) values
--('1004','MQWF34Handler','AAM_6.0_MQWF34_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.MQWFXMLAgent','MQWF 3.4 Action Service Handler', 0);
--INSERT INTO $SCHEMA$.ACT_HNDLR_T
--(ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) values
--('1005','MQWF35Handler','AAM_6.0_MQWF35PLUS_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.MQWFwa0cAgent','MQWF 3.5+ Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1006','CellHandler','AAM_6.0_CELL_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.CellEmailHandler','Cell Email Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1007','PagerHandler','AAM_6.0_PAGER_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.PagerEmailHandler','Pager Email Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1008','SCAHandler','AAM_6.0_SCA','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.ScaHandler','SCA Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('2001','IBMSimpleE2ADeterminerHandler','AAM_6.0_PLUGSVC','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.determination.SimpleEventToActionDeterminer','Pluggable Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('2002','IBMSimpleA2ADeterminerHandler','AAM_6.0_PLUGSVC','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.determination.SimpleActionToActionDeterminer','Pluggable Service Handler', 0);

INSERT INTO $SCHEMA$.PLUG_SVC_T
  (ID,NAME,SERVICE_TYPE,SUB_TYPE,ACTION_HANDLER_ID,DESCRIPTION, VERSION_ID) 
VALUES ('1001','IBMSimpleE2ADeterminer','DETERMINER','E2A.IBM_SIMPLE','2001','Simple E2A Determiner', 0);

INSERT INTO $SCHEMA$.PLUG_SVC_T
  (ID,NAME,SERVICE_TYPE,SUB_TYPE,ACTION_HANDLER_ID,DESCRIPTION, VERSION_ID) 
VALUES ('1002','IBMSimpleA2ADeterminer','DETERMINER','A2A.IBM_SIMPLE','2002','Simple A2A Determiner', 0);

----------- REST DATA SERVICES SECURITY & AUTHORIZATION TABLES
----------- TABLES
CREATE TABLE $SCHEMA$.RAT_RESOURCE_GROUPS  
(
  ID VARCHAR(128) NOT NULL ,
  PARENT_ID VARCHAR(128) ,
  DESCRIPTION VARCHAR(256)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.MON_AUTH_PK2 ON
  $SCHEMA$.RAT_RESOURCE_GROUPS (ID) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.RAT_AUTH_DOMAIN  
(
  RESOURCE1 VARCHAR(256) NOT NULL ,
  VERSION  DECIMAL(19) NOT NULL ,
  RESOURCE_GROUP VARCHAR(128) NOT NULL ,
  DESCRIPTION VARCHAR(256)
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.MON_AUTH_PK1 ON
  $SCHEMA$.RAT_AUTH_DOMAIN (RESOURCE1, VERSION) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.RAT_SUPERUSERS  
(
  USERID VARCHAR(256) NOT NULL ,
  ASSIGNED_BY VARCHAR(256) 
) IN $DBNAME$.DSTS32K;
  
COMMENT ON TABLE $SCHEMA$.RAT_SUPERUSERS IS 'ANY USER/GROUP ENTERED HERE WILL HAVE ALL ACCESS.';

CREATE UNIQUE INDEX $SCHEMA$.MON_AUTH_PK4 ON
  $SCHEMA$.RAT_SUPERUSERS (USERID) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.RAT_PARTY_ROLES
(
  USERID VARCHAR(256) NOT NULL ,
  ROLE VARCHAR(128) NOT NULL ,
  RESOURCE_GROUP VARCHAR(128) NOT NULL 
) IN $DBNAME$.DSTS32K;

COMMENT ON TABLE $SCHEMA$.RAT_PARTY_ROLES IS 'member roles in a resource group';

CREATE UNIQUE INDEX $SCHEMA$.MON_AUTH_PK5 ON
  $SCHEMA$.RAT_PARTY_ROLES (USERID, ROLE, RESOURCE_GROUP) USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.RAT_UPDATE_MONITOR  
(
  TABLE_NAME VARCHAR(40) NOT NULL ,
  MARKER VARCHAR(4) NOT NULL 
) IN $DBNAME$.DSTS32K;

COMMENT ON COLUMN $SCHEMA$.RAT_UPDATE_MONITOR.MARKER IS 'OLD or NEW';

COMMENT ON COLUMN $SCHEMA$.RAT_UPDATE_MONITOR.TABLE_NAME IS 'stores table names RAT_AUTH_ADMIN, RAT_PARTY_ROLES and RAT_RESOURCE_GROUPS';

CREATE UNIQUE INDEX $SCHEMA$.MON_AUTH_PK3 ON
  $SCHEMA$.RAT_UPDATE_MONITOR (TABLE_NAME) USING STOGROUP $STOGRP$;

INSERT INTO   $SCHEMA$.RAT_UPDATE_MONITOR VALUES ('RAT_AUTH_DOMAIN','OLD');
INSERT INTO   $SCHEMA$.RAT_UPDATE_MONITOR VALUES ('RAT_RESOURCE_GROUPS','OLD');
INSERT INTO   $SCHEMA$.RAT_UPDATE_MONITOR VALUES ('RAT_PARTY_ROLES','OLD');
INSERT INTO   $SCHEMA$.RAT_RESOURCE_GROUPS  VALUES('root','root','The default resource group');
----------- TRIGGERS
----------- REST DATA SERVICES SECURITY & AUTHORIZATION TRIGGERS
-----------

CREATE TRIGGER $SCHEMA$.MON_DELETE1
  AFTER DELETE ON $SCHEMA$.RAT_PARTY_ROLES
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
    
CREATE TRIGGER $SCHEMA$.MON_DELETE2
  AFTER DELETE ON $SCHEMA$.RAT_AUTH_DOMAIN
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  
CREATE TRIGGER $SCHEMA$.MON_DELETE3
  AFTER DELETE ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';

CREATE TRIGGER $SCHEMA$.MON_DELETE4
  AFTER DELETE ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  DELETE FROM $SCHEMA$.RAT_PARTY_ROLES WHERE RESOURCE_GROUP = OLD.ID; 
  
CREATE TRIGGER $SCHEMA$.MON_INSERT1
  AFTER INSERT ON $SCHEMA$.RAT_PARTY_ROLES
  REFERENCING NEW AS NEW
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';

CREATE TRIGGER $SCHEMA$.MON_INSERT2
  AFTER INSERT ON $SCHEMA$.RAT_AUTH_DOMAIN
  REFERENCING NEW AS NEW
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
 
CREATE TRIGGER $SCHEMA$.MON_INSERT3
  AFTER INSERT ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING NEW AS NEW
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';
  
CREATE TRIGGER $SCHEMA$.MON_SU
  AFTER INSERT ON $SCHEMA$.RAT_SUPERUSERS
  REFERENCING NEW AS NEW
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  
CREATE TRIGGER $SCHEMA$.MON_SU2
  AFTER DELETE ON $SCHEMA$.RAT_SUPERUSERS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  
CREATE TRIGGER $SCHEMA$.MON_SU3
  AFTER UPDATE ON $SCHEMA$.RAT_SUPERUSERS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';  
  
CREATE TRIGGER $SCHEMA$.MON_UPDATE1
  AFTER UPDATE ON $SCHEMA$.RAT_PARTY_ROLES
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  
CREATE TRIGGER $SCHEMA$.MON_UPDATE2
  AFTER UPDATE ON $SCHEMA$.RAT_AUTH_DOMAIN
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  
CREATE TRIGGER $SCHEMA$.MON_UPDATE3
  AFTER UPDATE ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';

CREATE TRIGGER $SCHEMA$.MODEL_DEPLOY
  AFTER INSERT ON $SCHEMA$.META_MODEL_T
  REFERENCING NEW AS NEW
  FOR EACH ROW MODE DB2SQL
  WHEN (
   NOT EXISTS (SELECT 1 FROM $SCHEMA$.rat_auth_domain  where new.model_id = concat('/', $SCHEMA$.rat_auth_domain.resource1)  )
  )
  INSERT INTO $SCHEMA$.RAT_AUTH_DOMAIN VALUES(substr(new.model_id,2), new.version,'root','');
  
CREATE TRIGGER $SCHEMA$.MON_ON_DELETE
  AFTER DELETE ON $SCHEMA$.RAT_UPDATE_MONITOR
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  INSERT INTO $SCHEMA$.RAT_UPDATE_MONITOR VALUES(OLD.table_name,'NEW')   ;
 
-----------DASHBOARD DATA TABLE
CREATE TABLE $SCHEMA$.DASHBOARD_DATA_T
(
  DASHBOARD_CONFIG                   BLOB(1024K)                        NOT NULL ,
  DASHBOARD_ID                       VARCHAR(128)                       NOT NULL ,
  DASHBOARD_OWNER                    VARCHAR(128)                       NOT NULL ,
  DASHBOARD_NAME                     VARCHAR(128)                       NOT NULL ,
  DASHBOARD_SHARED_USERS             VARCHAR(512)                               ,
  PRIMARY KEY ( DASHBOARD_ID ))
  IN $DBNAME$.DSTS32K;
  
CREATE UNIQUE INDEX $SCHEMA$.DASHBOARD_DATA_P ON
  $SCHEMA$.DASHBOARD_DATA_T (DASHBOARD_ID) USING STOGROUP $STOGRP$;

-- auxiliary table for LOB column DASHBOARD_CONFIG
CREATE AUX TABLE $SCHEMA$.AT25A17
   IN $DBNAME$.DSLOB17
   STORES $SCHEMA$.DASHBOARD_DATA_T COLUMN DASHBOARD_CONFIG;
-- auxiliary index for LOB column DASHBOARD_CONFIG
CREATE UNIQUE INDEX $SCHEMA$.AT25A17I
  ON $SCHEMA$.AT25A17 USING STOGROUP $STOGRP$;

CREATE TABLE $SCHEMA$.USER_DATA_T
(
  USER_ID                   VARCHAR(128)                     NOT NULL ,
  USER_CONFIG               VARCHAR(4000)          ,             
  PRIMARY KEY ( USER_ID )
) IN $DBNAME$.DSTS32K;

CREATE UNIQUE INDEX $SCHEMA$.USER_DATA_P ON
  $SCHEMA$.USER_DATA_T (USER_ID) USING STOGROUP $STOGRP$;

CONNECT RESET;

   
