-- Licensed Materials - Property of IBM
-- 5724-M24
-- c Copyright IBM Corporation 2007
-- The source code for this program is not published or otherwise
-- divested of its trade secrets, irrespective of what has been
-- deposited with the U.S. Copyright Office.
-- ------------------------------------------------------------------------------------
--
-- Scriptfile to create database and schema (tables) for Apache Derby
--
---------------------------------------------------------------------------------------
-- Following variables need to be changed for customization before running this script: 
-- $DBNAME$ = the database name, eg MONITOR
-- $SCHEMA$ = the schema name, eg MONITOR
-- $TERRITORY$ = the locale for this database, eg EN_US
--
-- Run the script in the Derby Interactive JDBC (ij) scripting tool.
--
-- USAGE:   ij> run 'createDatabaseDerby.ddl';
--
-- IMPORTANT:  Please make sure the files DIM_TIME_DATA.del and STOPWATCH.jar are in 
--             the local directory.  This script requires these 2 files!
--
-- ------------------------------------------------------------------------------------
-- create the database and connect to it (connection url for embedded database)
CONNECT '$DBNAME$;create=true;territory=$TERRITORY$' as $DBNAME$;  

-------------------
-- Create tables --
-------------------


CREATE TABLE $SCHEMA$.SCHEMA_VERSION
(
  SCHEMA_VERSION                     INTEGER                           NOT NULL ,
  DATA_MIGRATION                     SMALLINT                          NOT NULL ,
  PRIMARY KEY ( SCHEMA_VERSION )
);

CREATE TABLE $SCHEMA$.UNIQUE_KEY_T
(
  NEXT_KEY                           INTEGER                           NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( NEXT_KEY )
);

CREATE TABLE $SCHEMA$.META_MODEL_UNVERSIONED_T
(
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  GENERAL_PROPERTIES                 BLOB(1000M)                                ,
  INTEGRATION_STATUS                 VARCHAR(32)                                ,
  INTEGRATION_TYPE                   VARCHAR(32)                                ,
  CEICELL                            VARCHAR(128)                               ,
  CEINODE                            VARCHAR(128)                               ,
  CEISERVER                          VARCHAR(128)                               ,
  CEICLUSTER                         VARCHAR(128)                               ,
  CEIHOST                            VARCHAR(128)                               ,
  CEIPORT                            VARCHAR(128)                               ,
  CEIUSER_ID                         VARCHAR(128)                               ,
  CEIPASSWORD                        VARCHAR(128)                               ,
  CEIEVENT_GROUP_PROFILE_LIST        VARCHAR(128)                               ,
  MEGA_NAME_MAPPER                   BLOB(1000M)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( MODEL_ID )
);

CREATE TABLE $SCHEMA$.META_MODEL_T
(
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  SCHEMA_NAME                        VARCHAR(128)                               ,
  DISPLAY_NAME                       VARCHAR(1000)                              ,
  CURRENT_STATE                      VARCHAR(32)                                ,
  INITIAL_STATE                      VARCHAR(32)                                ,
  TARGET_STATE                       VARCHAR(32)                                ,
  APPLICATION                        VARCHAR(64)                                ,
  MODEL                              BLOB(1000M)                                ,
  CUBES                              BLOB(1000M)                                ,
  DIAGRAMS                           BLOB(1000M)                                ,
  SCHEMA_DDLS                        BLOB(1000M)                                ,
  DMS_DDLS                           BLOB(1000M)                                ,
  GENERAL_PROPERTIES                 BLOB(1000M)                                ,
  DEPLOYMENT_OPTIONS                 BLOB(1000M)                                ,
  NAME_MAPPER                        BLOB(1000M)                                ,
  UDF_JARS                           BLOB(1000M)                                ,
  ABXHOST                            VARCHAR(128)                               ,
  ABXPORT                            INTEGER                                    ,
  ABXUSER_ID                         VARCHAR(128)                               ,
  ABXPASSWORD                        VARCHAR(128)                               ,
  INTEGRATION_STATUS                 VARCHAR(32)                                ,
  REPLACEMENT_VERSION_DATE           BIGINT                                     ,
  IS_STOP_REQUESTED                  SMALLINT                                   ,
  APPLICATION_UPDATE                 SMALLINT                                   ,
  KPI_CACHE_REFRESH_INTERVAL         SMALLINT                                   ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( MODEL_ID, VERSION )
);

CREATE TABLE $SCHEMA$.META_MODEL_STEP_T
(
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  STEP_NAME                          VARCHAR(32)                       NOT NULL ,
  LAST_SUCCESS_RUN_TIME              TIMESTAMP                                  ,
  LAST_FAIL_RUN_TIME                 TIMESTAMP                                  ,
  IS_COMPLETE                        SMALLINT                                   ,
  MONITOR_VERSION                    VARCHAR(32)                                ,
  MESSAGE                            BLOB(1000M)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( MODEL_ID, VERSION, STEP_NAME )
);

CREATE TABLE $SCHEMA$.META_MONITOR_CONTEXT_T
(
  MCID                               VARCHAR(256)                      NOT NULL ,
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  MCPARENT_ID                        VARCHAR(256)                               ,
  DIAGRAM                            BLOB(1000M)                                ,
  DISPLAY_NAME                       VARCHAR(1000)                     NOT NULL ,
  CUBE_NAME                          VARCHAR(128)                      NOT NULL ,
  TABLE_NAME                         VARCHAR(128)                      NOT NULL ,
  VIEW_NAME                          VARCHAR(128)                      NOT NULL ,
  DMS_TABLE_NAME                     VARCHAR(128)                      NOT NULL ,
  DMS_CHG_LOG_TABLE                  VARCHAR(128)                      NOT NULL ,  
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( MCID, VERSION )
);

CREATE TABLE $SCHEMA$.META_MONITOR_METRIC_T
(
  METRIC_ID                          VARCHAR(384)                      NOT NULL ,
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  MCID                               VARCHAR(256)                      NOT NULL ,
  METRIC_TYPE                        VARCHAR(14)                       NOT NULL ,
  IS_SORTABLE                        SMALLINT                                   ,
  COLUMN_NAME                        VARCHAR(128)                      NOT NULL ,
  DISPLAY_NAME                       VARCHAR(1000)                     NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( METRIC_ID, VERSION )
);

CREATE TABLE $SCHEMA$.META_DIMENSION_T
(
  DIMENSION_ID                       VARCHAR(384)                      NOT NULL ,
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  MCID                               VARCHAR(256)                      NOT NULL ,
  TABLE_NAME                         VARCHAR(128)                               ,
  DISPLAY_NAME                       VARCHAR(1000)                     NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DIMENSION_ID, VERSION )
);

CREATE TABLE $SCHEMA$.META_DIM_ATTR_T
(
  DIM_ATTRIBUTE_ID                   VARCHAR(512)                      NOT NULL ,
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  DIMENSION_ID                       VARCHAR(384)                      NOT NULL ,
  DIM_LEVEL                          INTEGER                           NOT NULL ,
  IS_PART_OF_DIM_KEY                 INTEGER                           NOT NULL ,
  METRIC_ID                          VARCHAR(384)                      NOT NULL ,
  DISPLAY_NAME                       VARCHAR(1000)                     NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DIM_ATTRIBUTE_ID, VERSION )
);

CREATE TABLE $SCHEMA$.META_MEASURE_T
(
  MEASURE_ID                         VARCHAR(384)                      NOT NULL ,
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  MCID                               VARCHAR(256)                      NOT NULL ,
  DISPLAY_NAME                       VARCHAR(1000)                     NOT NULL ,
  AGGREGATION_TYPE                   VARCHAR(14)                       NOT NULL ,
  TRACKING_KEY                       VARCHAR(256)                               ,
  MEASURE_DATA_TYPE                  VARCHAR(14)                                ,
  SOURCE_METRIC_ID                   VARCHAR(384)                               ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( MEASURE_ID, VERSION )
);

CREATE TABLE $SCHEMA$.META_KPI_CONTEXT_T
(
  KPI_CONTEXT_ID                     VARCHAR(384)                      NOT NULL ,
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  DISPLAY_NAME                       VARCHAR(1000)                              ,
  DIAGRAM                            BLOB(1000M)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( KPI_CONTEXT_ID, VERSION )
);

CREATE TABLE $SCHEMA$.META_KPI_T
(
  KPI_ID                             VARCHAR(384)                      NOT NULL ,
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  KPI_CONTEXT_ID                     VARCHAR(256)                               ,
  VERSION                            BIGINT                            NOT NULL ,
  DISPLAY_NAME                       VARCHAR(1000)                              ,
  DESCRIPTION                        VARCHAR(256)                               ,
  KPI_ORIGIN                         SMALLINT                          NOT NULL ,
  KPI_DATA_TYPE                      SMALLINT                          NOT NULL ,
  TARGET                             DOUBLE                                     ,
  KPI_RANGE_TYPE                     SMALLINT                          NOT NULL ,
  KPI_CALC_METHOD                    SMALLINT                          NOT NULL ,
  AGGREGATED_MEASURE_ID              VARCHAR(384)                               ,
  AGGREGATED_METRIC_ID               VARCHAR(384)                               ,
  AGGREGATION_FUNCTION               VARCHAR(10)                                ,
  VERSION_AGGREGATION                SMALLINT                                   ,
  TIME_PERIOD_METRIC_ID              VARCHAR(384)                               ,
  TIME_PERIOD_METHOD                 SMALLINT                                   ,
  REPEATING_PERIOD_DURATION          SMALLINT                                   ,
  REPEATING_PERIOD_BASIS             SMALLINT                                   ,
  REPEATING_PERIOD_TIMEZONE          VARCHAR(30)                                ,
  ROLLING_PERIOD_DURATION            SMALLINT                                   ,
  ROLLING_PERIOD_QUANTITY            INTEGER                                    ,
  FIXED_PERIOD_START                 TIMESTAMP                                  ,
  FIXED_PERIOD_END                   TIMESTAMP                                  ,
  FIXED_PERIOD_TIMEZONE              VARCHAR(30)                                ,
  CALCULATED_KPI_EXPRESSION          CLOB(1048576)                              ,
  USER_ID                            VARCHAR(256)                               ,
  VIEW_ACCESS                        SMALLINT                          NOT NULL ,
  FORMAT_DECIMAL_PRECISION           SMALLINT                                   ,
  FORMAT_CURRENCY                    VARCHAR(3)                                 ,
  FORMAT_PERCENTAGE                  SMALLINT                          NOT NULL ,
  KPI_CACHE_VALUE                    DOUBLE                                     ,
  KPI_LAST_CALCULATION_TIME          TIMESTAMP                                  ,
  KPI_CACHE_OVERRIDE_INTERVAL        SMALLINT                                   ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( KPI_ID, VERSION )
);

CREATE TABLE $SCHEMA$.META_KPI_METRIC_FILTER_T
(
  KPI_METRIC_FILTER_ID               VARCHAR(512)                      NOT NULL ,
  KPI_ID                             VARCHAR(384)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  FILTER_METRIC_ID                   VARCHAR(512)                      NOT NULL ,
  FILTER_OPERATOR                    SMALLINT                          NOT NULL ,
  FILTER_OPERATOR_CASE_SENSITIVE     SMALLINT                          NOT NULL ,
  FILTER_VALUE                       CLOB(1048576)                              ,
  VALUE_INDEX                        INTEGER                           NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( KPI_METRIC_FILTER_ID, VERSION, VALUE_INDEX )
);

CREATE TABLE $SCHEMA$.META_KPI_RANGE_T
(
  KPI_RANGE_ID                       VARCHAR(512)                      NOT NULL ,
  KPI_ID                             VARCHAR(384)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  DISPLAY_NAME                       VARCHAR(1000)                              ,
  START_VALUE                        DOUBLE                            NOT NULL ,
  END_VALUE                          DOUBLE                            NOT NULL ,
  COLOR                              VARCHAR(7)                                 ,
  ICON                               VARCHAR(256)                               ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( KPI_RANGE_ID, VERSION )
);

CREATE TABLE $SCHEMA$.META_KPI_DEPENDENCY_T
(
  PARENT_KPI_ID                      VARCHAR(384)                      NOT NULL ,
  PARENT_KPI_VERSION                 BIGINT                            NOT NULL ,
  CHILD_KPI_ID                       VARCHAR(384)                      NOT NULL ,
  CHILD_KPI_VERSION                  BIGINT                            NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( PARENT_KPI_ID, PARENT_KPI_VERSION, CHILD_KPI_ID, CHILD_KPI_VERSION )
);

CREATE TABLE $SCHEMA$.KEY_HELPER_T
(
  TABLE_NAME                         VARCHAR(128)                      NOT NULL ,
  NEXT_KEY                           INTEGER                           NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( TABLE_NAME )
);

CREATE TABLE $SCHEMA$.DMS_METADATA_T
(
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  MCID                               VARCHAR(256)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  DMS_COPY_PROC                      VARCHAR(128)                               ,
  TARGET_TABLE                       VARCHAR(128)                               ,
  LAST_TIME_STARTED                  BIGINT                                     ,
  LAST_TIME_CMPLTD                   BIGINT                                     ,
  DMS_INTERVAL                       BIGINT                                     ,
  PROGRESS_FLAG                      SMALLINT                                   ,
  IS_NEW_FLAG                        SMALLINT                                   ,
  SUSPENDED_FLAG                     SMALLINT                                   ,
  IS_DELETED_FLAG                    SMALLINT                                   ,
  INVOKE_NOW                         SMALLINT                                   ,
  ROWS_COPIED                        INTEGER                                    ,
  ROWS_PRUNED                        INTEGER                                    ,
  DMS_COPY_PROC_MESG                 VARCHAR(4000)                               ,
  VERSION_ID                         SMALLINT                           NOT NULL ,
  PRIMARY KEY ( MODEL_ID, MCID, VERSION )
);

CREATE TABLE $SCHEMA$.DMS_HISTORY_T
(
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  MCID                               VARCHAR(256)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  TIME_STARTED                       BIGINT                            NOT NULL ,
  TIME_COMPLETED                     BIGINT                                     ,
  DMS_INTERVAL                       BIGINT                                     ,
  ROWS_COPIED                        INTEGER                                    ,
  ROWS_PRUNED                        INTEGER                                    ,
  DMS_COPY_PROC_MESG                 VARCHAR(4000)                               ,
  IS_DELETED_FLAG                    SMALLINT                                    ,
  VERSION_ID                         SMALLINT                           NOT NULL ,
  PRIMARY KEY ( MODEL_ID, MCID, VERSION, TIME_STARTED )
);

CREATE TABLE $SCHEMA$.META_OUTBOUND_EVENT_T
(
  MODEL_ID                           VARCHAR(128)                      NOT NULL ,
  EVENT_ID                           VARCHAR(384)                      NOT NULL ,
  CONTEXT_ID                         VARCHAR(256)                      NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  DISPLAY_NAME                       VARCHAR(1000)                              ,
  EXTENSION_NAME                     VARCHAR(1000)                              ,
  CONTEXT_TYPE                       VARCHAR(100)                               ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( EVENT_ID, VERSION )
);

CREATE TABLE $SCHEMA$.ACT_MGR_ALERTS_T
(
  ALERT_ID                           VARCHAR(32)                       NOT NULL ,
  CONTEXT_DEF                        VARCHAR(300)                               ,
  CONTEXT_INST_ID                    VARCHAR(32)                                ,
  MODEL_NAME                         VARCHAR(300)                               ,
  MODEL_ID                           VARCHAR(128)                               ,
  CONTEXT_NAME                       VARCHAR(300)                               ,
  CONTEXT_ID                         VARCHAR(256)                               ,
  MODEL_VERSION                      VARCHAR(128)                               ,
  BUS_SIT_NAME                       VARCHAR(254)                      NOT NULL ,
  CBE_XML                            BLOB(1000M)                                ,
  ALERT_SUBJECT                      VARCHAR(240)                      NOT NULL ,
  ALERT_BODY                         CLOB(1048576)                     NOT NULL ,
  ALERT_TIMESTAMP                    TIMESTAMP                                  ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ALERT_ID )
);

CREATE TABLE $SCHEMA$.DASHBOARD_ALERT_T
(
  USER_ID                            VARCHAR(32)                       NOT NULL ,
  ALERT_ID                           VARCHAR(32)                       NOT NULL ,
  ACKNOWLEDGED                       SMALLINT                          NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( USER_ID, ALERT_ID )
);
CREATE INDEX DASHBOARD_ALERT_I1 ON $SCHEMA$.DASHBOARD_ALERT_T (USER_ID);

CREATE TABLE $SCHEMA$.ACT_HNDLR_T
(
  ID                                 VARCHAR(32)                       NOT NULL ,
  NAME                               VARCHAR(64)                       NOT NULL ,
  TYPE                               VARCHAR(64)                       NOT NULL ,
  INVOCATION_TYPE                    VARCHAR(20)                                ,
  INVOCATION_PROFILE                 VARCHAR(1024)                              ,
  DESCRIPTION                        VARCHAR(1024)                              ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ID )
);

CREATE TABLE $SCHEMA$.ACTION_HANDLER_CTRL_T
(
  ACTION_ID                          VARCHAR(32)                       NOT NULL ,
  HANDLER_ID                         VARCHAR(32)                       NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ACTION_ID, HANDLER_ID )
);
CREATE INDEX ACTN_HNDLR_CTRL_I1 ON $SCHEMA$.ACTION_HANDLER_CTRL_T (ACTION_ID);

CREATE TABLE $SCHEMA$.PLUG_SVC_T
(
  ID                                 VARCHAR(32)                       NOT NULL ,
  NAME                               VARCHAR(64)                       NOT NULL ,
  SERVICE_TYPE                       VARCHAR(20)                       NOT NULL ,
  SUB_TYPE                           VARCHAR(20)                       NOT NULL ,
  ACTION_HANDLER_ID                  VARCHAR(32)                       NOT NULL ,
  DESCRIPTION                        VARCHAR(1024)                              ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ID )
);

CREATE TABLE $SCHEMA$.ACT_TMPLT_T
(
  ID                                 VARCHAR(32)                       NOT NULL ,
  NAME                               VARCHAR(256)                      NOT NULL ,
  TYPE                               VARCHAR(20)                       NOT NULL ,
  CONTENT                            BLOB(1024K)                       NOT NULL ,
  DESCRIPTION                        VARCHAR(1024)                              ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ID )
);
CREATE INDEX ACT_TMPLT_I1 ON $SCHEMA$.ACT_TMPLT_T (NAME);
CREATE INDEX ACT_TMPLT_I2 ON $SCHEMA$.ACT_TMPLT_T (TYPE);


CREATE TABLE $SCHEMA$.ACTION_T
(
  ID                                 VARCHAR(32)                       NOT NULL ,
  NAME                               VARCHAR(64)                       NOT NULL ,
  TYPE                               VARCHAR(64)                       NOT NULL ,
  PARAMETER_SET                      BLOB(1024K)                                ,
  STATE                              VARCHAR(20)                                ,
  ALIAS_NAME                         VARCHAR(64)                                ,
  HANDLER_ID                         VARCHAR(32)                                ,
  TEMPLATE_ID                        VARCHAR(32)                                ,
  DESCRIPTION                        VARCHAR(1024)                              ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ID )
);

ALTER TABLE $SCHEMA$.ACTION_HANDLER_CTRL_T
    ADD CONSTRAINT D_ACTION_ID       FOREIGN KEY (ACTION_ID)
                                         REFERENCES $SCHEMA$.ACTION_T(ID) ON DELETE CASCADE;

CREATE TABLE $SCHEMA$.SUBSCRIBED_USER_T
(
  USER_ID                            VARCHAR(32)                       NOT NULL ,
  ETAACTRL_ID                        VARCHAR(32)                       NOT NULL ,
  DASHBOARD                          SMALLINT                          NOT NULL ,
  EMAIL                              SMALLINT                          NOT NULL ,
  CELL                               SMALLINT                          NOT NULL ,
  PAGER                              SMALLINT                          NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( USER_ID, ETAACTRL_ID )
);
CREATE INDEX SUBSCRIBED_USER_I1 ON $SCHEMA$.SUBSCRIBED_USER_T (USER_ID);
CREATE INDEX SUBSCRIBED_USER_I2 ON $SCHEMA$.SUBSCRIBED_USER_T (ETAACTRL_ID);

CREATE TABLE $SCHEMA$.ETAMATCH_T
(
  ID                                 VARCHAR(32)                       NOT NULL ,
  DETERMINER_TYPE                    VARCHAR(20)                       NOT NULL ,
  EXTENSION_NAME                     VARCHAR(256)                       NOT NULL ,
  DESCRIPTION                        VARCHAR(1024)                              ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ID )
);
CREATE INDEX ETAMATCH_I1 ON $SCHEMA$.ETAMATCH_T (EXTENSION_NAME);

CREATE TABLE $SCHEMA$.ATAMATCH_T
(
  ID                                 VARCHAR(32)                       NOT NULL ,
  DETERMINER_TYPE                    VARCHAR(20)                       NOT NULL ,
  PARENT_ACTION_ID                   VARCHAR(32)                       NOT NULL ,
  CHILD_ACTION_ID                    VARCHAR(32)                       NOT NULL ,
  DESCRIPTION                        VARCHAR(1024)                              ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ID )
);

CREATE TABLE $SCHEMA$.ETAACTRL_T
(
  ID                                 VARCHAR(32)                       NOT NULL ,
  ETAMID                             VARCHAR(32)                       NOT NULL ,
  ACTION_ID                          VARCHAR(32)                       NOT NULL ,
  NAME                               VARCHAR(264)                      NOT NULL ,
  DESCRIPTION                        VARCHAR(1024)                              ,
  BINDING_TIMESTAMP                  TIMESTAMP                                  ,
  CATEGORY                           VARCHAR(264)                               ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ID )
);
CREATE INDEX ETAACTRL_I1 ON $SCHEMA$.ETAACTRL_T (ETAMID);
CREATE INDEX ETAACTRL_I2 ON $SCHEMA$.ETAACTRL_T (ACTION_ID);
CREATE INDEX ETAACTRL_I3 ON $SCHEMA$.ETAACTRL_T (ETAMID, ACTION_ID);


CREATE TABLE $SCHEMA$.ACTION_MANAGER_PROP_T
(
  ACTION_PROP_NAME                   VARCHAR(100)                      NOT NULL ,
  ACTION_PROP_VALUE                  VARCHAR(240)                      NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ACTION_PROP_NAME )
);


CREATE TABLE $SCHEMA$.EQ_MODEL_VERSION_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  MODEL                              VARCHAR(1000)                     NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  FAIL_TIME                          BIGINT                                     ,
  LAST_SUB_INST                      VARCHAR(32)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID )
);
CREATE INDEX EQ_MODEL_VER_I1 ON $SCHEMA$.EQ_MODEL_VERSION_T (MODEL, VERSION);

CREATE TABLE $SCHEMA$.DQ_MODEL_VERSION_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  MODEL                              VARCHAR(1000)                     NOT NULL ,
  VERSION                            BIGINT                            NOT NULL ,
  FAIL_TIME                          BIGINT                                     ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID )
);
CREATE INDEX DQ_MODEL_VER_I1 ON $SCHEMA$.DQ_MODEL_VERSION_T (MODEL, VERSION);

CREATE TABLE $SCHEMA$.EQ_INSTANCE_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  MV_DBID                            VARCHAR(32)                       NOT NULL ,
  ROOT_INST_ID                       VARCHAR(1000)                     NOT NULL ,
  FAIL_TIME                          BIGINT                                     ,
  LAST_SUB_TIME                      BIGINT                                     ,
  LAST_SUB_STATUS                    INTEGER                           NOT NULL ,
  LAST_SEQUENCE_NUM                  BIGINT                            NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID ),
  FOREIGN KEY ( MV_DBID ) REFERENCES $SCHEMA$.EQ_MODEL_VERSION_T( DBID ) ON DELETE CASCADE
);

CREATE TABLE $SCHEMA$.EQ_FAILED_EVENT_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  INSTANCE_DBID                      VARCHAR(32)                       NOT NULL ,
  GU_INST_ID                         VARCHAR(1000)                              ,
  EQ_SEQUENCE                        BIGINT                            NOT NULL ,
  EVENT_SEQUENCE                     VARCHAR(512)                               ,
  F_SUMMARY                          VARCHAR(256)                      NOT NULL ,
  F_DETAILS                          BLOB(1024K)                                ,
  F_TIME                             BIGINT                            NOT NULL ,
  CREATION_TIME                      BIGINT                                     ,
  EXT_NAME                           VARCHAR(64)                                ,
  CONSUMPTION_TIME                   BIGINT                            NOT NULL ,
  LAST_SUB_TIME                      BIGINT                                     ,
  LAST_SUB_STATUS                    INTEGER                           NOT NULL ,
  HISTORY                            BLOB(20000K)                               ,
  DATA                               BLOB(1000M)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID ),
  FOREIGN KEY ( INSTANCE_DBID ) REFERENCES $SCHEMA$.EQ_INSTANCE_T( DBID ) ON DELETE CASCADE
);
CREATE INDEX EQ_FAILED_EVENT_I1 ON $SCHEMA$.EQ_FAILED_EVENT_T (INSTANCE_DBID);
CREATE INDEX EQ_FAILED_EVENT_I2 ON $SCHEMA$.EQ_FAILED_EVENT_T (INSTANCE_DBID, EQ_SEQUENCE);

CREATE TABLE $SCHEMA$.DQ_DEAD_EVENT_T
(
  DBID                               VARCHAR(32)                       NOT NULL ,
  MV_DBID                            VARCHAR(32)                       NOT NULL ,
  F_SUMMARY                          VARCHAR(256)                      NOT NULL ,
  F_DETAILS                          BLOB(1024K)                                ,
  F_TIME                             BIGINT                            NOT NULL ,
  DATA                               BLOB(1000M)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( DBID ),
  FOREIGN KEY ( MV_DBID ) REFERENCES $SCHEMA$.DQ_MODEL_VERSION_T( DBID ) ON DELETE CASCADE
);
CREATE INDEX DQ_DEAD_EVENT_I1 ON $SCHEMA$.DQ_DEAD_EVENT_T (MV_DBID);

--
-- dim time
--

create table $SCHEMA$.DIM_TIME
  (SURROGATE_KEY DATE not null, 
   DAY_NUM SMALLINT not null, 
   MONTH_NUM SMALLINT not null,
   YEAR_NUM INTEGER not null, 
   primary key(SURROGATE_KEY));

CALL SYSCS_UTIL.SYSCS_IMPORT_DATA('MONITOR', 'DIM_TIME', null, null, 'dim_time_data.del', null, null, null, 0) ;

CREATE  UNIQUE INDEX  $SCHEMA$.IXDTIME ON $SCHEMA$.DIM_TIME ("YEAR_NUM" ASC, "MONTH_NUM" ASC, "DAY_NUM" ASC);

--
-- End dim time
--

-- set schema version to: 602
INSERT INTO $SCHEMA$.SCHEMA_VERSION (SCHEMA_VERSION, DATA_MIGRATION) VALUES( 61, 0 );


-- View: ActTmplt
CREATE VIEW $SCHEMA$.ACT_TMPLT(ID, NAME, TYPE, DESCRIPTION ) AS
 SELECT $SCHEMA$.ACT_TMPLT_T.ID, $SCHEMA$.ACT_TMPLT_T.NAME, $SCHEMA$.ACT_TMPLT_T.TYPE, $SCHEMA$.ACT_TMPLT_T.DESCRIPTION
 FROM $SCHEMA$.ACT_TMPLT_T;

-- View: ETAMatch
CREATE VIEW $SCHEMA$.ETAMATCH(ID, DETERMINER_TYPE, EXTENSION_NAME, DESCRIPTION ) AS
 SELECT $SCHEMA$.ETAMATCH_T.ID, $SCHEMA$.ETAMATCH_T.DETERMINER_TYPE, $SCHEMA$.ETAMATCH_T.EXTENSION_NAME, $SCHEMA$.ETAMATCH_T.DESCRIPTION
 FROM $SCHEMA$.ETAMATCH_T;

-- View: Action
CREATE VIEW $SCHEMA$.ACTION(ID, NAME, TYPE ) AS
 SELECT $SCHEMA$.ACTION_T.ID, $SCHEMA$.ACTION_T.NAME, $SCHEMA$.ACTION_T.TYPE
 FROM $SCHEMA$.ACTION_T;

-- View: ActMgrProp
CREATE VIEW $SCHEMA$.ACT_MGR_PROP(ACTION_PROP_NAME, ACTION_PROP_VALUE ) AS
 SELECT $SCHEMA$.ACTION_MANAGER_PROP_T.ACTION_PROP_NAME, $SCHEMA$.ACTION_MANAGER_PROP_T.ACTION_PROP_VALUE
 FROM $SCHEMA$.ACTION_MANAGER_PROP_T;

-- View: PlugSvc
CREATE VIEW $SCHEMA$.PLUG_SVC(ID, NAME, SERVICE_TYPE, SUB_TYPE, ACTION_HANDLER_ID ) AS
 SELECT $SCHEMA$.PLUG_SVC_T.ID, $SCHEMA$.PLUG_SVC_T.NAME, $SCHEMA$.PLUG_SVC_T.SERVICE_TYPE, $SCHEMA$.PLUG_SVC_T.SUB_TYPE, $SCHEMA$.PLUG_SVC_T.ACTION_HANDLER_ID
 FROM $SCHEMA$.PLUG_SVC_T;

-- View: Alerts
CREATE VIEW $SCHEMA$.ALERTS(ALERT_ID, BUS_SIT_NAME, ALERT_SUBJECT, ALERT_TIMESTAMP ) AS
 SELECT $SCHEMA$.ACT_MGR_ALERTS_T.ALERT_ID, $SCHEMA$.ACT_MGR_ALERTS_T.BUS_SIT_NAME, $SCHEMA$.ACT_MGR_ALERTS_T.ALERT_SUBJECT, $SCHEMA$.ACT_MGR_ALERTS_T.ALERT_TIMESTAMP
 FROM $SCHEMA$.ACT_MGR_ALERTS_T;

-- View: EQ
CREATE view $SCHEMA$.EQ_FE_IDS
(
  DBID,
  INSTANCE_DBID,
  EQ_SEQUENCE
)
AS SELECT DBID, INSTANCE_DBID, EQ_SEQUENCE FROM $SCHEMA$.EQ_FAILED_EVENT_T;

--
-- Populate Action Mgr Tables
--
INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Host','localhost', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Port','25', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Sender','default@127.0.0.1', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.UID.Field','uid', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.Email.Field','mail', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.CellEmail.Field','mobile', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.PagerEmail.Field','pager', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.CEI_Logging','no', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.URL','ldap://localhost:389', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.UserName','cn=root', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.Password','', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Hostname','localhost', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Port','1414', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Channel','FMCQM.CL.TCP', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.UID.Field','uid', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Email.Field','mail', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.CellEmail.Field','mobile', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.PagerEmail.Field','pager', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Provider','$VMM_HOST$:$VMM_PORT$', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Entity.Type','PersonAccount', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID)    
  VALUES ('ActionManager.VMM.Group.Type','Group', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1001','EmailHandler','AAM_6.0_EMAIL_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.EmailActionHandler','Email Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1002','AlertHandler','AAM_6.0_ALERT_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.AlertActionHandler','Alert Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1003','WebServiceHandler','AAM_6.0_WEBSERVICES_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.WebServicesHandler','WebService Action Service Handler', 0);

--To reactivate MQWF handlers uncomment these lines
--INSERT INTO $SCHEMA$.ACT_HNDLR_T
--(ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) values
--('1004','MQWF34Handler','AAM_6.0_MQWF34_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.MQWFXMLAgent','MQWF 3.4 Action Service Handler', 0);
--INSERT INTO $SCHEMA$.ACT_HNDLR_T
--(ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) values
--('1005','MQWF35Handler','AAM_6.0_MQWF35PLUS_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.MQWFwa0cAgent','MQWF 3.5+ Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1006','CellHandler','AAM_6.0_CELL_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.CellEmailHandler','Cell Email Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1007','PagerHandler','AAM_6.0_PAGER_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.PagerEmailHandler','Pager Email Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1008','SCAHandler','AAM_6.0_SCA','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.ScaHandler','SCA Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('2001','IBMSimpleE2ADeterminerHandler','AAM_6.0_PLUGSVC','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.determination.SimpleEventToActionDeterminer','Pluggable Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('2002','IBMSimpleA2ADeterminerHandler','AAM_6.0_PLUGSVC','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.determination.SimpleActionToActionDeterminer','Pluggable Service Handler', 0);

INSERT INTO $SCHEMA$.PLUG_SVC_T
  (ID,NAME,SERVICE_TYPE,SUB_TYPE,ACTION_HANDLER_ID,DESCRIPTION, VERSION_ID) 
VALUES ('1001','IBMSimpleE2ADeterminer','DETERMINER','E2A.IBM_SIMPLE','2001','Simple E2A Determiner', 0);

INSERT INTO $SCHEMA$.PLUG_SVC_T
  (ID,NAME,SERVICE_TYPE,SUB_TYPE,ACTION_HANDLER_ID,DESCRIPTION, VERSION_ID) 
VALUES ('1002','IBMSimpleA2ADeterminer','DETERMINER','A2A.IBM_SIMPLE','2002','Simple A2A Determiner', 0);

----------- REST DATA SERVICES SECURITY & AUTHORIZATION TABLES-------------

CREATE TABLE $SCHEMA$.RAT_RESOURCE_GROUPS  (
   ID VARCHAR(128) NOT NULL ,
    PARENT_ID VARCHAR(128) ,
    DESCRIPTION VARCHAR(256) 
);

ALTER TABLE $SCHEMA$.RAT_RESOURCE_GROUPS
    ADD CONSTRAINT MON_AUTH_PK2 PRIMARY KEY (ID);
                      
CREATE TABLE $SCHEMA$.RAT_AUTH_DOMAIN  (
   RESOURCE1 VARCHAR(256) NOT NULL ,
   VERSION  BIGINT NOT NULL ,
    RESOURCE_GROUP VARCHAR(128) NOT NULL ,
    DESCRIPTION VARCHAR(256) 
);

ALTER TABLE $SCHEMA$.RAT_AUTH_DOMAIN
    ADD CONSTRAINT MON_AUTH_PK1 PRIMARY KEY (RESOURCE1, VERSION);

CREATE TABLE $SCHEMA$.RAT_SUPERUSERS  (
   USERID VARCHAR(256) NOT NULL ,
    ASSIGNED_BY VARCHAR(256) 
);



ALTER TABLE $SCHEMA$.RAT_SUPERUSERS
   ADD CONSTRAINT MON_AUTH_PK4 PRIMARY KEY (USERID);

CREATE TABLE $SCHEMA$.RAT_PARTY_ROLES  (
   USERID VARCHAR(256) NOT NULL ,
   ROLE VARCHAR(128) NOT NULL ,
   RESOURCE_GROUP VARCHAR(128) NOT NULL 
);



ALTER TABLE $SCHEMA$.RAT_PARTY_ROLES
   ADD CONSTRAINT MON_AUTH_PK5 PRIMARY KEY (USERID, ROLE, RESOURCE_GROUP);

CREATE TABLE $SCHEMA$.RAT_UPDATE_MONITOR  (
   TABLE_NAME VARCHAR(40) NOT NULL ,
    MARKER VARCHAR(4) NOT NULL 
);

ALTER TABLE $SCHEMA$.RAT_UPDATE_MONITOR
   ADD CONSTRAINT MON_AUTH_PK3 PRIMARY KEY (TABLE_NAME);

INSERT INTO   $SCHEMA$.RAT_UPDATE_MONITOR VALUES ('RAT_AUTH_DOMAIN','OLD');
INSERT INTO   $SCHEMA$.RAT_UPDATE_MONITOR VALUES ('RAT_RESOURCE_GROUPS','OLD');
INSERT INTO   $SCHEMA$.RAT_UPDATE_MONITOR VALUES ('RAT_PARTY_ROLES','OLD');
INSERT INTO   $SCHEMA$.RAT_RESOURCE_GROUPS  VALUES('root','root','The default resource group');


----------- REST DATA SERVICES SECURITY & AUTHORIZATION TRIGGERS
CREATE TRIGGER $SCHEMA$.MON_DELETE1
  AFTER DELETE ON $SCHEMA$.RAT_PARTY_ROLES
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
    
CREATE TRIGGER $SCHEMA$.MON_DELETE2
  AFTER DELETE ON $SCHEMA$.RAT_AUTH_DOMAIN
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  
CREATE TRIGGER $SCHEMA$.MON_DELETE3
  AFTER DELETE ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';

CREATE TRIGGER $SCHEMA$.MON_DELETE4
  AFTER DELETE ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  DELETE FROM $SCHEMA$.RAT_PARTY_ROLES WHERE RESOURCE_GROUP = OLD.ID; 
  
CREATE TRIGGER $SCHEMA$.MON_INSERT1
  AFTER INSERT ON $SCHEMA$.RAT_PARTY_ROLES
  REFERENCING NEW AS NEW
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  
CREATE TRIGGER $SCHEMA$.MON_INSERT2
  AFTER INSERT ON $SCHEMA$.RAT_AUTH_DOMAIN
  REFERENCING NEW AS NEW
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
    
CREATE TRIGGER $SCHEMA$.MON_INSERT3
  AFTER INSERT ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING NEW AS NEW
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';
    
CREATE TRIGGER $SCHEMA$.MON_SU
  AFTER INSERT ON $SCHEMA$.RAT_SUPERUSERS
  REFERENCING NEW AS NEW
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  
CREATE TRIGGER $SCHEMA$.MON_SU2
  AFTER DELETE ON $SCHEMA$.RAT_SUPERUSERS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  
CREATE TRIGGER $SCHEMA$.MON_SU3
  AFTER UPDATE ON $SCHEMA$.RAT_SUPERUSERS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  
CREATE TRIGGER $SCHEMA$.MON_UPDATE1
  AFTER UPDATE ON $SCHEMA$.RAT_PARTY_ROLES
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  
CREATE TRIGGER $SCHEMA$.MON_UPDATE2
  AFTER UPDATE ON $SCHEMA$.RAT_AUTH_DOMAIN
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  
CREATE TRIGGER $SCHEMA$.MON_UPDATE3
  AFTER UPDATE ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';

CREATE TRIGGER $SCHEMA$.MON_ON_DELETE
  AFTER DELETE ON $SCHEMA$.RAT_UPDATE_MONITOR
  REFERENCING OLD AS OLD
  FOR EACH ROW MODE DB2SQL
  insert into $SCHEMA$.RAT_UPDATE_MONITOR values(OLD.table_name,'NEW')   ;

-----------DASHBOARD DATA TABLE
CREATE TABLE $SCHEMA$.DASHBOARD_DATA_T
(
  DASHBOARD_CONFIG                   BLOB(1024K)                        NOT NULL ,
  DASHBOARD_ID                       VARCHAR(128)                       NOT NULL ,
  DASHBOARD_OWNER                    VARCHAR(128)                       NOT NULL ,
  DASHBOARD_NAME                     VARCHAR(128)                       NOT NULL ,
  DASHBOARD_SHARED_USERS             VARCHAR(512)                               ,
  PRIMARY KEY ( DASHBOARD_ID )
);

CREATE TABLE $SCHEMA$.USER_DATA_T
(
  USER_ID                   VARCHAR(128)                     NOT NULL ,
  USER_CONFIG               VARCHAR(4096)          ,             
  PRIMARY KEY ( USER_ID )
);

--
--
-- user-defined functions DAYS() and MIDNIGHT_SECONDS()
--
--

call sqlj.install_jar('stopwatch.jar','$SCHEMA$.stopwatch',0);
   
CALL SYSCS_UTIL.SYSCS_SET_DATABASE_PROPERTY   ('derby.database.classpath', '$SCHEMA$.stopwatch'); 
  
CREATE FUNCTION $SCHEMA$.TO_GMT_TIMESTAMP(t1 timestamp) returns TIMESTAMP parameter style java no sql language java external name 'stopwatch.Stopwatch.TO_GMT_TIMESTAMP'
   returns NULL ON NULL input;
  
CREATE FUNCTION $SCHEMA$.DAYS(t1 timestamp) returns INTEGER parameter style java no sql language java external name 'stopwatch.Stopwatch.DAYS'
   returns NULL ON NULL input;

CREATE FUNCTION $SCHEMA$.MIDNIGHT_SECONDS(t1 timestamp) returns INTEGER parameter style java no SQL language java external name 'stopwatch.Stopwatch.MIDNIGHT_SECONDS'
   returns NULL ON NULL input;


DISCONNECT;



