-- Licensed Materials - Property of IBM
-- 5724-M24
-- c Copyright IBM Corporation 2007
-- The source code for this program is not published or otherwise
-- divested of its trade secrets, irrespective of what has been
-- deposited with the U.S. Copyright Office.
-- ------------------------------------------------------------------------------------
--
-- Scriptfile to create database and schema (tables) for Oracle 10g
--
---------------------------------------------------------------------------------------
-- Following variables need to be changed for customization before running this script: 
-- $SCHEMA$ = the schema owner, eg MONITOR
-- $DBPASS$ = the schema owner password
-- $TSDIR$ = the directory or prefix for tablespace datafiles, eg DEFAULTTS
--
-- Run the script in SQL*Plus.
--
-- USAGE:  sqlplus system/passw0rd@orcl @createDatabaseOracle.ddl 
--
-- ------------------------------------------------------------------------------------

-------------------------
-- Create tablespaces 
-------------------------
CREATE TABLESPACE INSTANCE
  DATAFILE '$TSDIR$_INSTANCE.dbf' SIZE 500M AUTOEXTEND ON NEXT 100M MAXSIZE UNLIMITED LOGGING;

CREATE TABLESPACE DMSTS
  DATAFILE '$TSDIR$_DMSTS.dbf' SIZE 100M AUTOEXTEND ON NEXT 20M MAXSIZE UNLIMITED LOGGING;

CREATE TABLESPACE INDEXTS
  DATAFILE '$TSDIR$_INDEXTS.dbf' SIZE 250M AUTOEXTEND ON NEXT 50M MAXSIZE UNLIMITED LOGGING;

CREATE TABLESPACE LOBTS
  DATAFILE '$TSDIR$_LOBTS.dbf' SIZE 200M AUTOEXTEND ON NEXT 40M MAXSIZE UNLIMITED LOGGING;

-------------------------
-- Create schema owner --
-------------------------
CREATE USER $SCHEMA$ IDENTIFIED BY $DBPASS$;

GRANT ALL PRIVILEGES TO $SCHEMA$;

-------------------
-- Create tables --
-------------------

CREATE TABLE $SCHEMA$.SCHEMA_VERSION
(
  SCHEMA_VERSION                     NUMBER(10,0)        NOT NULL ,
  DATA_MIGRATION                     NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE 
  LOGGING;

ALTER TABLE $SCHEMA$.SCHEMA_VERSION ADD
(
  PRIMARY KEY ( SCHEMA_VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.UNIQUE_KEY_T
(
  NEXT_KEY                           NUMBER(10,0)        NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.UNIQUE_KEY_T ADD
(
  PRIMARY KEY ( NEXT_KEY )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_MODEL_UNVERSIONED_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  GENERAL_PROPERTIES                 BLOB                         ,
  INTEGRATION_STATUS                 VARCHAR2(32)                 ,
  INTEGRATION_TYPE                   VARCHAR2(32)                 ,
  CEICELL                            VARCHAR2(128)                ,
  CEINODE                            VARCHAR2(128)                ,
  CEISERVER                          VARCHAR2(128)                ,
  CEICLUSTER                         VARCHAR2(128)                ,
  CEIHOST                            VARCHAR2(128)                ,
  CEIPORT                            VARCHAR2(128)                ,
  CEIUSER_ID                         VARCHAR2(128)                ,
  CEIPASSWORD                        VARCHAR2(128)                ,
  CEIEVENT_GROUP_PROFILE_LIST        VARCHAR2(128)                ,
  MEGA_NAME_MAPPER                   BLOB                         ,   
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_MODEL_UNVERSIONED_T ADD
(
  PRIMARY KEY ( MODEL_ID )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_MODEL_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  SCHEMA_NAME                        VARCHAR2(128)                ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  CURRENT_STATE                      VARCHAR2(32)                 ,
  INITIAL_STATE                      VARCHAR2(32)                 ,
  TARGET_STATE                       VARCHAR2(32)                 ,
  APPLICATION                        VARCHAR2(64)                 ,
  MODEL                              BLOB                         ,
  CUBES                              BLOB                         ,
  DIAGRAMS                           BLOB                         ,
  SCHEMA_DDLS                        BLOB                         ,
  DMS_DDLS                           BLOB                         ,
  GENERAL_PROPERTIES                 BLOB                         ,
  DEPLOYMENT_OPTIONS                 BLOB                         ,
  NAME_MAPPER                        BLOB                         ,
  UDF_JARS                           BLOB                         ,
  ABXHOST                            VARCHAR2(128)                ,
  ABXPORT                            NUMBER(10,0)                 ,
  ABXUSER_ID                         VARCHAR2(128)                ,
  ABXPASSWORD                        VARCHAR2(128)                ,
  INTEGRATION_STATUS                 VARCHAR2(32)                 ,
  REPLACEMENT_VERSION_DATE           NUMBER(20,0)                 ,
  IS_STOP_REQUESTED                  NUMBER(5,0)                  ,
  APPLICATION_UPDATE                 NUMBER(5,0)                  ,
  KPI_CACHE_REFRESH_INTERVAL         NUMBER(5,0)                  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_MODEL_T ADD
(
  PRIMARY KEY ( MODEL_ID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_MODEL_STEP_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  STEP_NAME                          VARCHAR2(32)        NOT NULL ,
  LAST_SUCCESS_RUN_TIME              TIMESTAMP                    ,
  LAST_FAIL_RUN_TIME                 TIMESTAMP                    ,
  IS_COMPLETE                        NUMBER(5,0)                  ,
  MONITOR_VERSION                    VARCHAR2(32)                 ,
  MESSAGE                            BLOB                         ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_MODEL_STEP_T ADD
(
  PRIMARY KEY ( MODEL_ID, VERSION, STEP_NAME )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_MONITOR_CONTEXT_T
(
  MCID                               VARCHAR2(256)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  MCPARENT_ID                        VARCHAR2(256)                ,
  DIAGRAM                            BLOB                         ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  CUBE_NAME                          VARCHAR2(128)       NOT NULL ,
  TABLE_NAME                         VARCHAR2(128)       NOT NULL ,
  VIEW_NAME                          VARCHAR2(128)       NOT NULL ,
  DMS_TABLE_NAME                     VARCHAR2(128)       NOT NULL ,
  DMS_CHG_LOG_TABLE                  VARCHAR2(128)       NOT NULL ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_MONITOR_CONTEXT_T ADD
(
  PRIMARY KEY ( MCID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_MONITOR_METRIC_T
(
  METRIC_ID                          VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  METRIC_TYPE                        VARCHAR2(14)        NOT NULL ,
  IS_SORTABLE                        NUMBER(5,0)                  ,
  COLUMN_NAME                        VARCHAR2(128)       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_MONITOR_METRIC_T ADD
(
  PRIMARY KEY ( METRIC_ID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_DIMENSION_T
(
  DIMENSION_ID                       VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  TABLE_NAME                         VARCHAR2(128)                ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_DIMENSION_T ADD
(
  PRIMARY KEY ( DIMENSION_ID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_DIM_ATTR_T
(
  DIM_ATTRIBUTE_ID                   VARCHAR2(512)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DIMENSION_ID                       VARCHAR2(384)       NOT NULL ,
  DIM_LEVEL                          NUMBER(10,0)        NOT NULL ,
  IS_PART_OF_DIM_KEY                 NUMBER(10,0)        NOT NULL ,
  METRIC_ID                          VARCHAR2(384)       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_DIM_ATTR_T ADD
(
  PRIMARY KEY ( DIM_ATTRIBUTE_ID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_MEASURE_T
(
  MEASURE_ID                         VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  AGGREGATION_TYPE                   VARCHAR2(14)        NOT NULL ,
  TRACKING_KEY                       VARCHAR2(256)                ,
  MEASURE_DATA_TYPE                  VARCHAR2(14)                 ,
  SOURCE_METRIC_ID                   VARCHAR2(384)                ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_MEASURE_T ADD
(
  PRIMARY KEY ( MEASURE_ID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_KPI_CONTEXT_T
(
  KPI_CONTEXT_ID                     VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  DIAGRAM                            BLOB                         ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_KPI_CONTEXT_T ADD
(
  PRIMARY KEY ( KPI_CONTEXT_ID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_KPI_T
(
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  KPI_CONTEXT_ID                     VARCHAR2(256)                ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  DESCRIPTION                        VARCHAR2(256)                ,
  KPI_ORIGIN                         NUMBER(5,0)         NOT NULL ,
  KPI_DATA_TYPE                      NUMBER(5,0)         NOT NULL ,
  TARGET                             NUMBER                       ,
  KPI_RANGE_TYPE                     NUMBER(5,0)         NOT NULL ,
  KPI_CALC_METHOD                    NUMBER(5,0)         NOT NULL ,
  AGGREGATED_MEASURE_ID              VARCHAR2(384)                ,
  AGGREGATED_METRIC_ID               VARCHAR2(384)                ,
  AGGREGATION_FUNCTION               VARCHAR2(10)                 ,
  VERSION_AGGREGATION                NUMBER(5,0)                  ,
  TIME_PERIOD_METRIC_ID              VARCHAR2(384)                ,
  TIME_PERIOD_METHOD                 NUMBER(5,0)                  ,
  REPEATING_PERIOD_DURATION          NUMBER(5,0)                  ,
  REPEATING_PERIOD_BASIS             NUMBER(5,0)                  ,
  REPEATING_PERIOD_TIMEZONE          VARCHAR2(30)                 ,
  ROLLING_PERIOD_DURATION            NUMBER(5,0)                  ,
  ROLLING_PERIOD_QUANTITY            NUMBER(10,0)                 ,
  FIXED_PERIOD_START                 TIMESTAMP                    ,
  FIXED_PERIOD_END                   TIMESTAMP                    ,
  FIXED_PERIOD_TIMEZONE              VARCHAR2(30)                 ,
  CALCULATED_KPI_EXPRESSION          CLOB                         ,
  USER_ID                            VARCHAR2(256)                ,
  VIEW_ACCESS                        NUMBER(5,0)         NOT NULL ,
  FORMAT_DECIMAL_PRECISION           NUMBER(5,0)                  ,
  FORMAT_CURRENCY                    VARCHAR2(3)                  ,
  FORMAT_PERCENTAGE                  NUMBER(5,0)         NOT NULL ,
  KPI_CACHE_VALUE                    NUMBER                       ,
  KPI_LAST_CALCULATION_TIME          TIMESTAMP                    ,
  KPI_CACHE_OVERRIDE_INTERVAL        NUMBER(5,0)                  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING
  LOB (CALCULATED_KPI_EXPRESSION) STORE AS
  (
    TABLESPACE LOBTS
    ENABLE STORAGE IN ROW
    NOCACHE
  );

ALTER TABLE $SCHEMA$.META_KPI_T ADD
(
  PRIMARY KEY ( KPI_ID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_KPI_METRIC_FILTER_T
(
  KPI_METRIC_FILTER_ID               VARCHAR2(512)       NOT NULL ,
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  FILTER_METRIC_ID                   VARCHAR2(512)       NOT NULL ,
  FILTER_OPERATOR                    NUMBER(5,0)         NOT NULL ,
  FILTER_OPERATOR_CASE_SENSITIVE     NUMBER(5,0)         NOT NULL ,
  FILTER_VALUE                       CLOB                         ,
  VALUE_INDEX                        NUMBER(10,0)        NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING
  LOB (FILTER_VALUE) STORE AS
  (
    TABLESPACE LOBTS
    ENABLE STORAGE IN ROW
    NOCACHE
  );

ALTER TABLE $SCHEMA$.META_KPI_METRIC_FILTER_T ADD
(
  PRIMARY KEY ( KPI_METRIC_FILTER_ID, VERSION, VALUE_INDEX )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_KPI_RANGE_T
(
  KPI_RANGE_ID                       VARCHAR2(512)       NOT NULL ,
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  START_VALUE                        NUMBER              NOT NULL ,
  END_VALUE                          NUMBER              NOT NULL ,
  COLOR                              VARCHAR2(7)                  ,
  ICON                               VARCHAR2(256)                ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_KPI_RANGE_T ADD
(
  PRIMARY KEY ( KPI_RANGE_ID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_KPI_DEPENDENCY_T
(
  PARENT_KPI_ID                      VARCHAR2(384)       NOT NULL ,
  PARENT_KPI_VERSION                 NUMBER(20,0)        NOT NULL ,
  CHILD_KPI_ID                       VARCHAR2(384)       NOT NULL ,
  CHILD_KPI_VERSION                  NUMBER(20,0)        NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_KPI_DEPENDENCY_T ADD
(
  PRIMARY KEY ( PARENT_KPI_ID, PARENT_KPI_VERSION, CHILD_KPI_ID, CHILD_KPI_VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.KEY_HELPER_T
(
  TABLE_NAME                         VARCHAR2(128)       NOT NULL ,
  NEXT_KEY                           NUMBER(10,0)        NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.KEY_HELPER_T ADD
(
  PRIMARY KEY ( TABLE_NAME )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.DMS_METADATA_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DMS_COPY_PROC                      VARCHAR2(128)                ,
  TARGET_TABLE                       VARCHAR2(128)                ,
  LAST_TIME_STARTED                  NUMBER(20,0)                 ,
  LAST_TIME_CMPLTD                   NUMBER(20,0)                 ,
  DMS_INTERVAL                       NUMBER(20,0)                 ,
  PROGRESS_FLAG                      NUMBER(5,0)                  ,
  IS_NEW_FLAG                        NUMBER(5,0)                  ,
  SUSPENDED_FLAG                     NUMBER(5,0)                  ,
  IS_DELETED_FLAG                    NUMBER(5,0)                  ,
  INVOKE_NOW                         NUMBER(5,0)                  ,
  ROWS_COPIED                        NUMBER(10,0)                 ,
  ROWS_PRUNED                        NUMBER(10,0)                 ,
  DMS_COPY_PROC_MESG                 VARCHAR2(4000)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.DMS_METADATA_T ADD
(
  PRIMARY KEY ( MODEL_ID, MCID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.DMS_HISTORY_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  TIME_STARTED                       NUMBER(20,0)        NOT NULL ,
  TIME_COMPLETED                     NUMBER(20,0)                 ,
  DMS_INTERVAL                       NUMBER(20,0)                 ,
  ROWS_COPIED                        NUMBER(10,0)                 ,
  ROWS_PRUNED                        NUMBER(10,0)                 ,
  DMS_COPY_PROC_MESG                 VARCHAR2(4000)               ,
  IS_DELETED_FLAG                    NUMBER(5,0)                  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.DMS_HISTORY_T ADD
(
  PRIMARY KEY ( MODEL_ID, MCID, VERSION, TIME_STARTED )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.META_OUTBOUND_EVENT_T
(
  MODEL_ID                           VARCHAR2(128)                      NOT NULL ,
  EVENT_ID                           VARCHAR2(384)                      NOT NULL ,
  CONTEXT_ID                         VARCHAR2(256)                      NOT NULL ,
  VERSION                            NUMBER(20,0)                       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)                              ,
  EXTENSION_NAME                     VARCHAR2(1000)                              ,
  CONTEXT_TYPE                       VARCHAR2(100)                               ,
  VERSION_ID                         NUMBER(5,0)                         NOT NULL
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.META_OUTBOUND_EVENT_T ADD
(
  PRIMARY KEY ( EVENT_ID, VERSION )
  USING INDEX TABLESPACE INDEXTS
);



CREATE TABLE $SCHEMA$.ACT_MGR_ALERTS_T
(
  ALERT_ID                           VARCHAR2(32)        NOT NULL ,
  CONTEXT_DEF                        VARCHAR2(300)                ,
  CONTEXT_INST_ID                    VARCHAR2(32)                 ,
  MODEL_NAME                         VARCHAR2(300)                ,
  MODEL_ID                           VARCHAR2(128)                ,
  CONTEXT_NAME                       VARCHAR2(300)                ,
  CONTEXT_ID                         VARCHAR2(256)                ,
  MODEL_VERSION                      VARCHAR2(128)                ,
  BUS_SIT_NAME                       VARCHAR2(254)       NOT NULL ,
  CBE_XML                            BLOB                         ,
  ALERT_SUBJECT                      VARCHAR2(240)       NOT NULL ,
  ALERT_BODY                         CLOB                NOT NULL ,
  ALERT_TIMESTAMP                    TIMESTAMP                    ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING
  LOB (ALERT_BODY) STORE AS
  (
    TABLESPACE LOBTS
    ENABLE STORAGE IN ROW
    NOCACHE
  );

ALTER TABLE $SCHEMA$.ACT_MGR_ALERTS_T ADD
(
  PRIMARY KEY ( ALERT_ID )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.DASHBOARD_ALERT_T
(
  USER_ID                            VARCHAR2(32)        NOT NULL ,
  ALERT_ID                           VARCHAR2(32)        NOT NULL ,
  ACKNOWLEDGED                       NUMBER(5,0)         NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.DASHBOARD_ALERT_T ADD
(
  PRIMARY KEY ( USER_ID, ALERT_ID )
  USING INDEX TABLESPACE INDEXTS
);
CREATE INDEX DASHBOARD_ALERT_I1 ON $SCHEMA$.DASHBOARD_ALERT_T (USER_ID);

CREATE TABLE $SCHEMA$.ACT_HNDLR_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  NAME                               VARCHAR2(64)        NOT NULL ,
  TYPE                               VARCHAR2(64)        NOT NULL ,
  INVOCATION_TYPE                    VARCHAR2(20)                 ,
  INVOCATION_PROFILE                 VARCHAR2(1024)               ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.ACT_HNDLR_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.ACTION_HANDLER_CTRL_T
(
  ACTION_ID                          VARCHAR2(32)        NOT NULL ,
  HANDLER_ID                         VARCHAR2(32)        NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.ACTION_HANDLER_CTRL_T ADD
(
  PRIMARY KEY ( ACTION_ID, HANDLER_ID )
  USING INDEX TABLESPACE INDEXTS
);
CREATE INDEX ACTN_HNDLR_CTRL_I1 ON $SCHEMA$.ACTION_HANDLER_CTRL_T (ACTION_ID);

CREATE TABLE $SCHEMA$.PLUG_SVC_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  NAME                               VARCHAR2(64)        NOT NULL ,
  SERVICE_TYPE                       VARCHAR2(20)        NOT NULL ,
  SUB_TYPE                           VARCHAR2(20)        NOT NULL ,
  ACTION_HANDLER_ID                  VARCHAR2(32)        NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.PLUG_SVC_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.ACT_TMPLT_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  NAME                               VARCHAR2(256)       NOT NULL ,
  TYPE                               VARCHAR2(20)        NOT NULL ,
  CONTENT                            BLOB                NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.ACT_TMPLT_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE INDEXTS
);
CREATE INDEX ACT_TMPLT_I1 ON $SCHEMA$.ACT_TMPLT_T (NAME);
CREATE INDEX ACT_TMPLT_I2 ON $SCHEMA$.ACT_TMPLT_T (TYPE);


CREATE TABLE $SCHEMA$.ACTION_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  NAME                               VARCHAR2(64)        NOT NULL ,
  TYPE                               VARCHAR2(64)        NOT NULL ,
  PARAMETER_SET                      BLOB                         ,
  STATE                              VARCHAR2(20)                 ,
  ALIAS_NAME                         VARCHAR2(64)                 ,
  TEMPLATE_ID                        VARCHAR2(32)                 ,
  HANDLER_ID                         VARCHAR2(32)                 ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.ACTION_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.SUBSCRIBED_USER_T
(
  USER_ID                            VARCHAR2(32)        NOT NULL ,
  ETAACTRL_ID                        VARCHAR2(32)        NOT NULL ,
  DASHBOARD                          NUMBER(5,0)         NOT NULL ,
  EMAIL                              NUMBER(5,0)         NOT NULL ,
  CELL                               NUMBER(5,0)         NOT NULL ,
  PAGER                              NUMBER(5,0)         NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.SUBSCRIBED_USER_T ADD
(
  PRIMARY KEY ( USER_ID, ETAACTRL_ID )
  USING INDEX TABLESPACE INDEXTS
);
CREATE INDEX SUBSCRIBED_USER_I1 ON $SCHEMA$.SUBSCRIBED_USER_T (USER_ID);
CREATE INDEX SUBSCRIBED_USER_I2 ON $SCHEMA$.SUBSCRIBED_USER_T (ETAACTRL_ID);

CREATE TABLE $SCHEMA$.ETAMATCH_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  DETERMINER_TYPE                    VARCHAR2(20)        NOT NULL ,
  EXTENSION_NAME                     VARCHAR2(256)       NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.ETAMATCH_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE INDEXTS
);
CREATE INDEX ETAMATCH_I1 ON $SCHEMA$.ETAMATCH_T (EXTENSION_NAME);

CREATE TABLE $SCHEMA$.ATAMATCH_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  DETERMINER_TYPE                    VARCHAR2(20)        NOT NULL ,
  PARENT_ACTION_ID                   VARCHAR2(32)        NOT NULL ,
  CHILD_ACTION_ID                    VARCHAR2(32)        NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.ATAMATCH_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.ETAACTRL_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  ETAMID                             VARCHAR2(32)        NOT NULL ,
  ACTION_ID                          VARCHAR2(32)        NOT NULL ,
  NAME                               VARCHAR2(264)       NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  BINDING_TIMESTAMP                  TIMESTAMP                    ,
  CATEGORY                           VARCHAR2(264)                ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.ETAACTRL_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE INDEXTS
);
CREATE INDEX ETAACTRL_I1 ON $SCHEMA$.ETAACTRL_T (ETAMID);
CREATE INDEX ETAACTRL_I2 ON $SCHEMA$.ETAACTRL_T (ACTION_ID);
CREATE INDEX ETAACTRL_I3 ON $SCHEMA$.ETAACTRL_T (ETAMID, ACTION_ID);


CREATE TABLE $SCHEMA$.ACTION_MANAGER_PROP_T
(
  ACTION_PROP_NAME                   VARCHAR2(100)       NOT NULL ,
  ACTION_PROP_VALUE                  VARCHAR2(240)       NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.ACTION_MANAGER_PROP_T ADD
(
  PRIMARY KEY ( ACTION_PROP_NAME )
  USING INDEX TABLESPACE INDEXTS
);

CREATE TABLE $SCHEMA$.EQ_MODEL_VERSION_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  MODEL                              VARCHAR2(1000)                     NOT NULL ,
  VERSION                            NUMBER(20,0)                       NOT NULL ,
  FAIL_TIME                          NUMBER(20,0)                                ,
  LAST_SUB_INST                      VARCHAR2(32)                                ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID )
);
CREATE INDEX EQ_MODEL_VER_I1 ON $SCHEMA$.EQ_MODEL_VERSION_T (MODEL, VERSION);

CREATE TABLE $SCHEMA$.DQ_MODEL_VERSION_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  MODEL                              VARCHAR2(1000)                     NOT NULL ,
  VERSION                            NUMBER(20,0)                       NOT NULL ,
  FAIL_TIME                          NUMBER(20,0)                                ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID )
);
CREATE INDEX DQ_MODEL_VER_I1 ON $SCHEMA$.DQ_MODEL_VERSION_T (MODEL, VERSION);

CREATE TABLE $SCHEMA$.EQ_INSTANCE_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  MV_DBID                            VARCHAR2(32)                       NOT NULL ,
  ROOT_INST_ID                       VARCHAR2(1000)                     NOT NULL ,
  FAIL_TIME                          NUMBER(20,0)                                ,
  LAST_SUB_TIME                      NUMBER(20,0)                                ,
  LAST_SUB_STATUS                    NUMBER(10,0)                       NOT NULL ,
  LAST_SEQUENCE_NUM                  NUMBER(20,0)                       NOT NULL ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID ),
  FOREIGN KEY ( MV_DBID ) REFERENCES $SCHEMA$.EQ_MODEL_VERSION_T( DBID ) ON DELETE CASCADE
);

CREATE TABLE $SCHEMA$.EQ_FAILED_EVENT_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  INSTANCE_DBID                      VARCHAR2(32)                       NOT NULL ,
  GU_INST_ID                         VARCHAR2(1000)                              ,
  EQ_SEQUENCE                        NUMBER(20,0)                       NOT NULL ,
  EVENT_SEQUENCE                     VARCHAR2(512)                               ,
  F_SUMMARY                          VARCHAR2(256)                      NOT NULL ,
  F_DETAILS                          BLOB                                        ,
  F_TIME                             NUMBER(20,0)                       NOT NULL ,
  CREATION_TIME                      NUMBER(20,0)                                ,
  EXT_NAME                           VARCHAR2(64)                                ,
  CONSUMPTION_TIME                   NUMBER(20,0)                       NOT NULL ,
  LAST_SUB_TIME                      NUMBER(20,0)                                ,
  LAST_SUB_STATUS                    NUMBER(10,0)                       NOT NULL ,
  HISTORY                            BLOB                                        ,
  DATA                               BLOB                                        ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID ),
  FOREIGN KEY ( INSTANCE_DBID ) REFERENCES $SCHEMA$.EQ_INSTANCE_T( DBID ) ON DELETE CASCADE
);
CREATE INDEX EQ_FAILED_EVENT_I1 ON $SCHEMA$.EQ_FAILED_EVENT_T (INSTANCE_DBID);
CREATE INDEX EQ_FAILED_EVENT_I2 ON $SCHEMA$.EQ_FAILED_EVENT_T (INSTANCE_DBID, EQ_SEQUENCE);

CREATE TABLE $SCHEMA$.DQ_DEAD_EVENT_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  MV_DBID                            VARCHAR2(32)                       NOT NULL ,
  F_SUMMARY                          VARCHAR2(256)                      NOT NULL ,
  F_DETAILS                          BLOB                                        ,
  F_TIME                             NUMBER(20,0)                       NOT NULL ,
  DATA                               BLOB                                        ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID ),
  FOREIGN KEY ( MV_DBID ) REFERENCES $SCHEMA$.DQ_MODEL_VERSION_T( DBID ) ON DELETE CASCADE
);
CREATE INDEX DQ_DEAD_EVENT_I1 ON $SCHEMA$.DQ_DEAD_EVENT_T (MV_DBID);

--
-- dim time
--

CREATE TABLE $SCHEMA$.DIM_TIME
(
  SURROGATE_KEY                      DATE                NOT NULL ,
  DAY_NUM                            NUMBER(5,0)         NOT NULL ,
  MONTH_NUM                          NUMBER(5,0)         NOT NULL ,
  YEAR_NUM                           NUMBER(10,0)        NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

DECLARE
  start_date DATE;
  sk NUMBER;
  
  end_date DATE;
BEGIN
  start_date := to_date('01012000','DDMMYYYY');
  end_date := to_date('31122014','DDMMYYYY');
  DELETE FROM $SCHEMA$.DIM_TIME;
  WHILE(start_date <= end_date) LOOP
    INSERT INTO $SCHEMA$.DIM_TIME(surrogate_key,year_num,month_num,day_num) values (start_date,to_number(to_char(start_date,'YYYY')),to_number(to_char(start_date,'MM')),to_number(to_char(start_date,'DD')));
    start_date := start_date + 1;
  END LOOP;
  INSERT INTO $SCHEMA$.DIM_TIME(surrogate_key,year_num,month_num,day_num) values (TO_DATE('01019999','DDMMYYYY'),9999,1,1);
END;
.
/

ALTER TABLE $SCHEMA$.DIM_TIME ADD
(
  PRIMARY KEY ( SURROGATE_KEY )
  USING INDEX TABLESPACE INDEXTS
);

-- set schema version to: 602
INSERT INTO $SCHEMA$.SCHEMA_VERSION (SCHEMA_VERSION, DATA_MIGRATION) VALUES( 61, 0 );

-- View: ActTmplt
CREATE VIEW $SCHEMA$.ACT_TMPLT(ID, NAME, TYPE, DESCRIPTION ) AS 
 SELECT ACT_TMPLT_T.ID, ACT_TMPLT_T.NAME, ACT_TMPLT_T.TYPE, ACT_TMPLT_T.DESCRIPTION 
 FROM $SCHEMA$.ACT_TMPLT_T;
 
-- View: ETAMatch
CREATE VIEW $SCHEMA$.ETAMATCH(ID, DETERMINER_TYPE, EXTENSION_NAME, DESCRIPTION ) AS 
 SELECT ETAMATCH_T.ID, ETAMATCH_T.DETERMINER_TYPE, ETAMATCH_T.EXTENSION_NAME, ETAMATCH_T.DESCRIPTION 
 FROM $SCHEMA$.ETAMATCH_T;
 
-- View: Action
CREATE VIEW $SCHEMA$.ACTION(ID, NAME, TYPE ) AS 
 SELECT ACTION_T.ID, ACTION_T.NAME, ACTION_T.TYPE 
 FROM $SCHEMA$.ACTION_T;

-- View: ActMgrProp
CREATE VIEW $SCHEMA$.ACT_MGR_PROP(ACTION_PROP_NAME, ACTION_PROP_VALUE ) AS 
 SELECT ACTION_MANAGER_PROP_T.ACTION_PROP_NAME, ACTION_MANAGER_PROP_T.ACTION_PROP_VALUE 
 FROM $SCHEMA$.ACTION_MANAGER_PROP_T; 

-- View: PlugSvc
CREATE VIEW $SCHEMA$.PLUG_SVC(ID, NAME, SERVICE_TYPE, SUB_TYPE, ACTION_HANDLER_ID ) AS 
 SELECT PLUG_SVC_T.ID, PLUG_SVC_T.NAME, PLUG_SVC_T.SERVICE_TYPE, PLUG_SVC_T.SUB_TYPE, PLUG_SVC_T.ACTION_HANDLER_ID 
 FROM $SCHEMA$.PLUG_SVC_T; 

-- View: Alerts
CREATE VIEW $SCHEMA$.ALERTS(ALERT_ID, BUS_SIT_NAME, ALERT_SUBJECT, ALERT_TIMESTAMP ) AS 
 SELECT ACT_MGR_ALERTS_T.ALERT_ID, ACT_MGR_ALERTS_T.BUS_SIT_NAME, ACT_MGR_ALERTS_T.ALERT_SUBJECT, ACT_MGR_ALERTS_T.ALERT_TIMESTAMP 
 FROM $SCHEMA$.ACT_MGR_ALERTS_T; 

-- View: EQ
CREATE view $SCHEMA$.EQ_FE_IDS
(
  DBID,
  INSTANCE_DBID,
  EQ_SEQUENCE
)
AS SELECT DBID, INSTANCE_DBID, EQ_SEQUENCE FROM $SCHEMA$.EQ_FAILED_EVENT_T;

--
-- Populate Action Mgr Tables
--
INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Host','localhost', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Port','25', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Sender','default@127.0.0.1', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.UID.Field','uid', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.Email.Field','mail', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.CellEmail.Field','mobile', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.PagerEmail.Field','pager', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.CEI_Logging','no', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.URL','ldap://localhost:389', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.UserName','cn=root', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.Password',' ', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Hostname','localhost', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Port','1414', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Channel','FMCQM.CL.TCP', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.UID.Field','uid', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Email.Field','mail', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.CellEmail.Field','mobile', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.PagerEmail.Field','pager', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Provider','$VMM_HOST$:$VMM_PORT$', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Entity.Type','PersonAccount', 0);

INSERT INTO $SCHEMA$.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID)    
  VALUES ('ActionManager.VMM.Group.Type','Group', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1001','EmailHandler','AAM_6.0_EMAIL_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.EmailActionHandler','Email Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1002','AlertHandler','AAM_6.0_ALERT_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.AlertActionHandler','Alert Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1003','WebServiceHandler','AAM_6.0_WEBSERVICES_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.WebServicesHandler','WebService Action Service Handler', 0);

--To reactivate MQWF handlers uncomment these lines
--INSERT INTO $SCHEMA$.ACT_HNDLR_T
--(ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) values
--('1004','MQWF34Handler','AAM_6.0_MQWF34_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.MQWFXMLAgent','MQWF 3.4 Action Service Handler', 0);
--INSERT INTO $SCHEMA$.ACT_HNDLR_T
--(ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) values
--('1005','MQWF35Handler','AAM_6.0_MQWF35PLUS_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.MQWFwa0cAgent','MQWF 3.5+ Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1006','CellHandler','AAM_6.0_CELL_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.CellEmailHandler','Cell Email Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1007','PagerHandler','AAM_6.0_PAGER_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.PagerEmailHandler','Pager Email Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1008','SCAHandler','AAM_6.0_SCA','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.ScaHandler','SCA Action Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('2001','IBMSimpleE2ADeterminerHandler','AAM_6.0_PLUGSVC','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.determination.SimpleEventToActionDeterminer','Pluggable Service Handler', 0);

INSERT INTO $SCHEMA$.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('2002','IBMSimpleA2ADeterminerHandler','AAM_6.0_PLUGSVC','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.determination.SimpleActionToActionDeterminer','Pluggable Service Handler', 0);

INSERT INTO $SCHEMA$.PLUG_SVC_T
  (ID,NAME,SERVICE_TYPE,SUB_TYPE,ACTION_HANDLER_ID,DESCRIPTION, VERSION_ID) 
VALUES ('1001','IBMSimpleE2ADeterminer','DETERMINER','E2A.IBM_SIMPLE','2001','Simple E2A Determiner', 0);

INSERT INTO $SCHEMA$.PLUG_SVC_T
  (ID,NAME,SERVICE_TYPE,SUB_TYPE,ACTION_HANDLER_ID,DESCRIPTION, VERSION_ID) 
VALUES ('1002','IBMSimpleA2ADeterminer','DETERMINER','A2A.IBM_SIMPLE','2002','Simple A2A Determiner', 0);

----------- REST DATA SERVICES SECURITY & AUTHORIZATION TABLES

CREATE TABLE $SCHEMA$.RAT_RESOURCE_GROUPS  (
  ID VARCHAR2(128) NOT NULL ,
  PARENT_ID VARCHAR2(128) ,
  DESCRIPTION VARCHAR2(256) 
) TABLESPACE INSTANCE
  LOGGING;
  
ALTER TABLE $SCHEMA$.RAT_RESOURCE_GROUPS
        ADD CONSTRAINT MON_AUTH_PK2 PRIMARY KEY (ID);

CREATE TABLE $SCHEMA$.RAT_AUTH_DOMAIN  (
  RESOURCE1 VARCHAR2(256) NOT NULL ,
  VERSION  NUMBER(20) NOT NULL ,
  RESOURCE_GROUP VARCHAR2(128) NOT NULL ,
  DESCRIPTION VARCHAR2(256) 
) TABLESPACE INSTANCE
  LOGGING;

ALTER TABLE $SCHEMA$.RAT_AUTH_DOMAIN
        ADD CONSTRAINT MON_AUTH_PK1 PRIMARY KEY (RESOURCE1, VERSION);

CREATE TABLE $SCHEMA$.RAT_SUPERUSERS  (
  USERID VARCHAR2(256) NOT NULL ,
  ASSIGNED_BY VARCHAR2(256) 
) TABLESPACE INSTANCE
  LOGGING;

COMMENT ON TABLE $SCHEMA$.RAT_SUPERUSERS IS 'ANY USER/GROUP ENTERED HERE WILL HAVE ALL ACCESS.';

ALTER TABLE $SCHEMA$.RAT_SUPERUSERS
        ADD CONSTRAINT MON_AUTH_PK4 PRIMARY KEY (USERID);

CREATE TABLE $SCHEMA$.RAT_PARTY_ROLES  (
  USERID VARCHAR2(256) NOT NULL ,
  ROLE VARCHAR2(128) NOT NULL ,
  RESOURCE_GROUP VARCHAR2(128) NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

COMMENT ON TABLE $SCHEMA$.RAT_PARTY_ROLES IS 'member roles in a resource group';

ALTER TABLE $SCHEMA$.RAT_PARTY_ROLES
        ADD CONSTRAINT MON_AUTH_PK5 PRIMARY KEY (USERID, ROLE, RESOURCE_GROUP);

CREATE TABLE $SCHEMA$.RAT_UPDATE_MONITOR  (
   TABLE_NAME VARCHAR2(40) NOT NULL ,
   MARKER VARCHAR2(4) NOT NULL 
) TABLESPACE INSTANCE
  LOGGING;

COMMENT ON COLUMN $SCHEMA$.RAT_UPDATE_MONITOR.MARKER IS 'OLD or NEW';

COMMENT ON COLUMN $SCHEMA$.RAT_UPDATE_MONITOR.TABLE_NAME IS 'stores table
names RAT_AUTH_ADMIN, RAT_PARTY_ROLES and RAT_RESOURCE_GROUPS';

ALTER TABLE $SCHEMA$.RAT_UPDATE_MONITOR
        ADD CONSTRAINT MON_AUTH_PK3 PRIMARY KEY (TABLE_NAME);

INSERT INTO   $SCHEMA$.RAT_UPDATE_MONITOR VALUES ('RAT_AUTH_DOMAIN','OLD');
INSERT INTO   $SCHEMA$.RAT_UPDATE_MONITOR VALUES ('RAT_RESOURCE_GROUPS','OLD');
INSERT INTO   $SCHEMA$.RAT_UPDATE_MONITOR VALUES ('RAT_PARTY_ROLES','OLD');
INSERT INTO   $SCHEMA$.RAT_RESOURCE_GROUPS  VALUES('root','root','The default resource group');

----------- TRIGGERS
----------- REST DATA SERVICES SECURITY & AUTHORIZATION TRIGGERS
-----------


CREATE TRIGGER $SCHEMA$.MON_DELETE1
  AFTER DELETE ON $SCHEMA$.RAT_PARTY_ROLES
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
   
CREATE TRIGGER $SCHEMA$.MON_DELETE2
  AFTER DELETE ON $SCHEMA$.RAT_AUTH_DOMAIN
  REFERENCING OLD AS OLD
  FOR EACH ROW  
  BEGIN
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_DELETE3
  AFTER DELETE ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';
  END;
.
RUN;

CREATE TRIGGER $SCHEMA$.MON_DELETE4
  AFTER DELETE ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  DELETE FROM $SCHEMA$.RAT_PARTY_ROLES WHERE RESOURCE_GROUP=:OLD.ID; 
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_INSERT1
  AFTER INSERT ON $SCHEMA$.RAT_PARTY_ROLES
  REFERENCING NEW AS NEW
  FOR EACH ROW 
  BEGIN
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_INSERT2
  AFTER INSERT ON $SCHEMA$.RAT_AUTH_DOMAIN
  REFERENCING NEW AS NEW
  FOR EACH ROW 
  BEGIN
  UPDATE RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_INSERT3
  AFTER INSERT ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING NEW AS NEW
  FOR EACH ROW 
  BEGIN 
  UPDATE RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_SU
  AFTER INSERT ON $SCHEMA$.RAT_SUPERUSERS
  REFERENCING NEW AS NEW
  FOR EACH ROW
  BEGIN 
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_SU2
  AFTER DELETE ON $SCHEMA$.RAT_SUPERUSERS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_SU3
  AFTER UPDATE ON $SCHEMA$.RAT_SUPERUSERS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_UPDATE1
  AFTER UPDATE ON $SCHEMA$.RAT_PARTY_ROLES
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_UPDATE2
  AFTER UPDATE ON $SCHEMA$.RAT_AUTH_DOMAIN
  REFERENCING OLD AS OLD
  FOR EACH ROW
  BEGIN
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  END;
.
RUN;
  
CREATE TRIGGER $SCHEMA$.MON_UPDATE3
  AFTER UPDATE ON $SCHEMA$.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE $SCHEMA$.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';
  END;
.
RUN;

CREATE TRIGGER $SCHEMA$.MODEL_DEPLOY
  AFTER INSERT ON $SCHEMA$.META_MODEL_T
  REFERENCING NEW AS NEW
  FOR EACH ROW 
  DECLARE i PLS_INTEGER;
  BEGIN
  SELECT COUNT(*) 
  INTO i 
  FROM $SCHEMA$.rat_auth_domain 
  WHERE :new.model_id = concat('/', $SCHEMA$.rat_auth_domain.resource1);
  IF i = 0 THEN
     INSERT INTO $SCHEMA$.rat_auth_domain values(substr(:new.model_id,2), :new.version,'root','');
  END IF;
  END;
.
RUN;
    
CREATE TRIGGER $SCHEMA$.MON_ON_DELETE
  AFTER DELETE ON $SCHEMA$.RAT_UPDATE_MONITOR
  REFERENCING OLD AS OLD
  FOR EACH ROW
  BEGIN
  INSERT INTO $SCHEMA$.RAT_UPDATE_MONITOR values(:OLD.table_name,'NEW');
  END;
.
RUN;

 
-----------DASHBOARD DATA TABLE
CREATE TABLE $SCHEMA$.DASHBOARD_DATA_T
(
  DASHBOARD_CONFIG                   BLOB                                NOT NULL ,
  DASHBOARD_ID                       VARCHAR2(128)                       NOT NULL ,
  DASHBOARD_OWNER                    VARCHAR2(128)                       NOT NULL ,
  DASHBOARD_NAME                     VARCHAR2(128)                       NOT NULL ,
  DASHBOARD_SHARED_USERS             VARCHAR2(512)                               ,
  PRIMARY KEY ( DASHBOARD_ID )
) TABLESPACE INSTANCE
  LOGGING;
  
CREATE TABLE $SCHEMA$.USER_DATA_T
(
  USER_ID                   VARCHAR(128)                     NOT NULL ,
  USER_CONFIG               VARCHAR(4000)          ,             
  PRIMARY KEY ( USER_ID )
);

-----------USER DEFINED FUNCTIONS
CREATE FUNCTION $SCHEMA$.DAYS (TIME2 IN TIMESTAMP) RETURN NUMBER DETERMINISTIC 
  AS
  TIME1   TIMESTAMP;
  DAYSRC  NUMBER;
  BEGIN
      SELECT TO_TIMESTAMP('01.01.0001:00:00:00','DD.MM.YYYY:HH24:MI:SS') INTO TIME1 FROM DUAL;
      SELECT TRUNC(TO_NUMBER(SUBSTR((TIME2-TIME1),1,INSTR(TIME2-TIME1,' ')))) INTO DAYSRC FROM DUAL; 
      RETURN DAYSRC;
  END;
.
RUN;

CREATE FUNCTION $SCHEMA$.MIDNIGHT_SECONDS (TIME2 IN TIMESTAMP) RETURN NUMBER DETERMINISTIC
  AS
  TIME1   TIMESTAMP;
  HH      NUMBER;
  MM      NUMBER;
  SS      NUMBER;
  BEGIN
      SELECT TO_TIMESTAMP('01.01.0001:00:00:00','DD.MM.YYYY:HH24:MI:SS') INTO TIME1 FROM DUAL;
      SELECT SUBSTR((TIME2-TIME1), INSTR((TIME2-TIME1),' ')+7,2) INTO SS FROM DUAL;
      SELECT SUBSTR((TIME2-TIME1), INSTR((TIME2-TIME1),' ')+4,2) INTO MM FROM DUAL; 
      SELECT SUBSTR((TIME2-TIME1), INSTR((TIME2-TIME1),' ')+1,2) INTO HH FROM DUAL;
      RETURN (HH * 3600) + (MM * 60) + SS;
  END;
.
RUN;

QUIT


